#
# Find the Intel MKL >= 11.0 library in the system
#

# This module defines
# MKL_ROOT_DIR, where the root of the MKL instalation can be found
# MKL_INCLUDE_DIR, where to find mkl.h, etc
# MKL_LIBRARIES, the libraries to link agains MKL
# FOUND_MKL, if false, do not try to use MKL

# Check for existing processor type variable
IF (NOT PROC_TARGET) 
    MESSAGE(FATAL_ERROR "Specify PROC_TARGET")
ENDIF (NOT PROC_TARGET)

# Look for the env. variables that should have been 
# registered by MKL
SET(MKL_ROOT $ENV{MKLROOT})

# If cannot be found, then the user has to specify the path manually
IF(MKL_ROOT)

    # Find include and library pathes
    FIND_PATH   (MKL_INCLUDE_DIR mkl.h PATH 
                ${MKL_ROOT}/include)
	IF (APPLE)                
	    FIND_LIBRARY(MKL_LIBRARY_1 mkl_intel             
	                PATH ${MKL_ROOT}/lib/${PROC_TARGET})
	    FIND_LIBRARY(MKL_LIBRARY_2 mkl_intel_thread             
	                PATH ${MKL_ROOT}/lib/${PROC_TARGET})
	    SET(MKL_LIBRARIES ${MKL_LIBRARY_1} ${MKL_LIBRARY_2})                        
    ELSE (APPLE)
    	FIND_LIBRARY(MKL_LIBRARIES mkl guide             
	                PATH ${MKL_ROOT}/lib/em${PROC_TARGET}t)
    ENDIF (APPLE)
    
    MESSAGE(${MKL_LIBRARIES})
    
    IF (MKL_INCLUDE_DIR)
       MESSAGE ("-- Found MKL include dir: " ${MKL_INCLUDE_DIR})
    ENDIF (MKL_INCLUDE_DIR)
    
    IF (MKL_LIBRARIES)
       MESSAGE ("-- Found MKL libraries: " ${MKL_LIBRARIES})
    ENDIF (MKL_LIBRARIES)    
    
    IF (MKL_INCLUDE_DIR AND MKL_LIBRARIES)
        SET (FOUND_MKL "ON")
    ELSE (MKL_INCLUDE_DIR AND MKL_LIBRARIES)    
        SET (FOUND_MKL "OFF")
    ENDIF (MKL_INCLUDE_DIR AND MKL_LIBRARIES)    
ELSE (MKL_ROOT)
    SET (FOUND_MKL "OFF")	    
ENDIF(MKL_ROOT)
