#! /usr/bin/env python



import sys
import os
from optparse import OptionParser

##########################
# Globals
##########################

bench_exec = "/home/serban/workspace/CUKr_MPI/build/src/bench_cg"
solver_file = "solver.perf.txt"
solver_maxit = 1000

##########################
# Functions
##########################

def read_solver_performance():
    f = open(solver_file, "r")
    perf = (f.readline().rstrip("\n\r"))
    return perf
    
##########################
# Main
##########################

# Declare the option parser
usage = "usage: %prog [options] matrix_file"
descr = "Extract CG performance for the given CSR matrix" 
parser = OptionParser(usage)
parser.set_description(descr)                      

parser.add_option("-d", "--gpu-double", dest="gpu_double", action="store_true", default = False,
                      help="GPU supports double precision [default: %default]")
parser.add_option("-p", "--pe", dest="cpu_cores", default = 4,
                      help="Choose maximum no. of cores [default: %default]")     

# Read arguments
(options, args) = parser.parse_args()
if (len(args) < 1): 
    parser.print_help()
    sys.exit(1) 

# Process arguments
matrix_file = args[0]
print "\nReading matrix from: %s" % matrix_file

# Run CPU tests in double MKL
cpu_double_list = []
for i in range(1, int(options.cpu_cores) + 1):
    cmd = bench_exec + " --matrix=" + matrix_file + " --solver-precision=double --double-precision-blas=MKL --solver-maxit=%d -p %d" % (solver_maxit, i)
    print "Running: " + cmd
    os.system(cmd)
    perf = read_solver_performance()
    cpu_double_list.append(perf)

# Run CPU tests in single Generic
cpu_single_list = []
for i in range(1, int(options.cpu_cores) + 1):
    cmd = bench_exec + " --matrix=" + matrix_file + " --solver-precision=single --single-precision-blas=Generic --solver-maxit=%d -p %d" % (solver_maxit, i)
    print "Running: " + cmd
    os.system(cmd)
    perf = read_solver_performance()
    cpu_single_list.append(perf)

# Run GPU tests in double 
if (options.gpu_double):
    cmd = bench_exec + " --matrix=" + matrix_file + " --solver-precision=double --double-precision-blas=GPUBLAS --solver-maxit=%d" % solver_maxit
    print "Running: " + cmd
    os.system(cmd)
    gpu_double_perf = read_solver_performance()
        
# Run GPU tests in qdouble 
cmd = bench_exec + " --matrix=" + matrix_file + " --solver-precision=qdouble --qdouble-precision-blas=GPUBLAS --solver-maxit=%d" % solver_maxit
print "Running: " + cmd
os.system(cmd)
gpu_qdouble_perf = read_solver_performance() 

# Run GPU tests in single 
cmd = bench_exec + " --matrix=" + matrix_file + " --solver-precision=single --single-precision-blas=GPUBLAS --solver-maxit=%d" % solver_maxit
print "Running: " + cmd
os.system(cmd)
gpu_single_perf = read_solver_performance()    

# Get matrix name and open result file
(path, name) = os.path.split(matrix_file)
(root, ext)  = os.path.splitext(name)
f = open(root + ".perf.txt", "w")

# Write performances
cpu_double_perf = max(cpu_double_list)
cpu_single_perf = max(cpu_single_list)

f.write(str(cpu_double_perf) + "\n")
f.write(str(cpu_single_perf) + "\n")
if (options.gpu_double):
    f.write(str(gpu_double_perf) + "\n")
else:
    f.write("n/a\n")
f.write(str(gpu_qdouble_perf) + "\n")
f.write(str(gpu_single_perf) + "\n")
f.close()

print "Done.\n"
