/**
 * @file blas/blas1.c
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to 
 * SP/MP/MPI BLAS1 routines.
 * 	 
 */

#include "blas1.h"


/**
 * \brief Dot product
 * \param blas The blas used for the operation
 * \param x,y The vectors
 */
CUKR_DOUBLE CukrDOT(CUKR_BLAS blas, CUKR_VECTOR x, CUKR_VECTOR y)
{	
	#ifdef CUKR_MPI
		return CukrDOT_MPI(blas, x, y);
	#else
		#ifdef CUKR_MP
			return CukrDOT_MP(blas, x, y);
		#else
			return CukrDOT_SP(blas, x, y);
		#endif
	#endif		
}

// ------------------------------------------------------------------- //

/**
 * \brief AXPY (ax + y)
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The vectors
 */
void CukrAXPY(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR x, CUKR_VECTOR y)
{
	#ifdef CUKR_MPI
		CukrAXPY_MPI(blas, a, x, y);
	#else
		#ifdef CUKR_MP
			CukrAXPY_MP(blas, a, x, y);
		#else
			CukrAXPY_SP(blas, a, x, y);
		#endif
	#endif			
}

// ------------------------------------------------------------------- //

/**
 * \brief AYPX (ay + x)
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The vectors
 */
void CukrAYPX(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR x, CUKR_VECTOR y)
{
	#ifdef CUKR_MPI
		CukrAYPX_MPI(blas, a, x, y);
	#else
		#ifdef CUKR_MP
			CukrAYPX_MP(blas, a, x, y);
		#else
			CukrAYPX_SP(blas, a, x, y);
		#endif
	#endif		
}

// ------------------------------------------------------------------- //

/**
 * \brief Vector scaling 
 * \param blas The blas used for the operation
 * \param a The scaling constant
 * \param x The vectors being scaled
 */
void CukrSCAL(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR x)
{
	#ifdef CUKR_MPI
		CukrSCAL_MPI(blas, a, x);
	#else
		#ifdef CUKR_MP
			CukrSCAL_MP(blas, a, x);
		#else
			CukrSCAL_SP(blas, a, x);
		#endif
	#endif		
}

// ------------------------------------------------------------------- //

/**
 * \brief Vector copy
 * \param blas The blas used for the operation
 * \param x The source vector
 * \param y The destination vector
 */
void CukrCOPY(CUKR_BLAS blas, CUKR_VECTOR x, CUKR_VECTOR y)
{
	#ifdef CUKR_MPI
		CukrCOPY_MPI(blas, x, y);
	#else
		#ifdef CUKR_MP
			CukrCOPY_MP(blas, x, y);
		#else
			CukrCOPY_SP(blas, x, y);
		#endif
	#endif		
}


