/**
 * @file blas/blas1.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to 
 * SP/MP/MPI BLAS1 routines.
 * 	 
 */

#ifndef CUKR_BLAS1_H_
#define CUKR_BLAS1_H_

#include "blas.h"
#include "../mat_vec/vector.h"

/* Decide which include file to use,
 * depending on defines set at built time.*/ 
#ifdef CUKR_MPI
	#include "mpi/blas1_mpi.h"
#else
	#ifdef CUKR_MP
		#include "mp/blas1_mp.h"
	#else
		#include "sp/blas1_sp.h"
	#endif
#endif

/**
 * \brief Dot product
 * \param blas The blas used for the operation
 * \param x,y The vectors
 */
CUKR_DOUBLE CukrDOT(CUKR_BLAS blas, CUKR_VECTOR x, CUKR_VECTOR y);

// ------------------------------------------------------------------- //

/**
 * \brief AXPY (ax + y)
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The vectors
 */
void CukrAXPY(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR x, CUKR_VECTOR y);

// ------------------------------------------------------------------- //

/**
 * \brief AYPX (ay + x)
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The vectors
 */
void CukrAYPX(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR x, CUKR_VECTOR y);

// ------------------------------------------------------------------- //

/**
 * \brief Vector scaling 
 * \param blas The blas used for the operation
 * \param a The scaling constant
 * \param x The vectors being scaled
 */
void CukrSCAL(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR x);

// ------------------------------------------------------------------- //

/**
 * \brief Vector copy
 * \param blas The blas used for the operation
 * \param x The source vector
 * \param y The destination vector
 */
void CukrCOPY(CUKR_BLAS blas, CUKR_VECTOR x, CUKR_VECTOR y);

// ------------------------------------------------------------------- //


#endif /*CUKR_BLAS_H_*/
