/**
 * @file blas/blas2.c
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to 
 * SP/MP/MPI BLAS2 routines.
 * 	 
 */

#include "blas2.h"


/**
 * \brief Sparse matrix vector multiplication
 * \param blas The blas used for the operation
 * \param alpha Scaling factor for Ax
 * \param mat The matrix
 * \param x Vector being multiplied
 * \param beta Scaling factor for y
 * \param y Result vector
 */ 
void CukrSPMV(CUKR_BLAS blas, CUKR_DOUBLE alpha, CUKR_MATRIX mat, 
              CUKR_VECTOR x, CUKR_DOUBLE beta, CUKR_VECTOR y)
{
	#ifdef CUKR_MPI
		CukrSPMV_MPI(blas, alpha, mat, x, beta, y);
	#else
		#ifdef CUKR_MP
			CukrSPMV_MP(blas, alpha, mat, x, beta, y);
		#else
			CukrSPMV_SP(blas, alpha, mat, x, beta, y);
		#endif
	#endif	
}              

