/**
 * @file blas/blas2.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to 
 * SP/MP/MPI BLAS2 routines.
 * 	 
 */

#ifndef CUKR_BLAS2_H_
#define CUKR_BLAS2_H_

#include "blas.h"
#include "../mat_vec/matrix.h"
#include "../mat_vec/vector.h"

/* Decide which include file to use,
 * depending on defines set at built time.*/ 
#ifdef CUKR_MPI
	#include "mpi/blas2_mpi.h"
#else
	#ifdef CUKR_MP
		#include "mp/blas2_mp.h"
	#else
		#include "sp/blas2_sp.h"
	#endif
#endif

/**
 * \brief Sparse matrix vector multiplication
 * \param blas The blas used for the operation
 * \param alpha Scaling factor for Ax
 * \param mat The matrix
 * \param x Vector being multiplied
 * \param beta Scaling factor for y
 * \param y Result vector
 */ 
void CukrSPMV(CUKR_BLAS blas, CUKR_DOUBLE alpha, CUKR_MATRIX mat, 
              CUKR_VECTOR x, CUKR_DOUBLE beta, CUKR_VECTOR y);


#endif /*CUKR_BLAS2_H_*/
