/**
 * @file cublas_wrappers.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains wrappers for cublas functions.
 * 
 * To keep the library in ANSI C and only
 * at little as possible in C++, wrappers
 * for the CUBLAS functions are defined.
 * 
 * A wrapper is basically a call to the
 * cublas function + calls to cuda functions
 * (e.g. cudaThreadSychronize) that need to
 * be compiled by the CUDA compiler. 
 * 
 */

#ifndef CUKR_CUBLAS_WRAPPERS_H_
#define CUKR_CUBLAS_WRAPPERS_H_

/**
 * \brief DDOT wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
double cuda_cublasDdot(int n, double* x, int incx, double* y, int incy);

/**
 * \brief SDOT wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
float cuda_cublasSdot(int n, float* x, int incx, float* y, int incy);

/**
 * \brief DAXPY wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
void cuda_cublasDaxpy(int n, double a, double *x, int incx, double *y, int incy);

/**
 * \brief SAXPY wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
void cuda_cublasSaxpy(int n, float a, float *x, int incx, float *y, int incy);


/**
 * \brief DSCAL wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
void cuda_cublasDscal(int n, double a, double *x, int incx);

/**
 * \brief SSCAL wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
void cuda_cublasSscal(int n, float a, float *x, int incx);


/**
 * \brief DCOPY wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
void cuda_cublasDcopy(int n, double *x, int incx, double *y, int incy);

/**
 * \brief SCOPY wrapper
 */
#ifdef __cplusplus
extern "C"
#endif  
void cuda_cublasScopy(int n, float *x, int incx, float *y, int incy);


#endif /*CUKR_CUBLAS_WRAPPERS_H_*/
