/**
 * @file kernel_daypx.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Defines a double precision AYPX kernel.
 * 
 * AYPX: Y := X + aY 
 * 
 * With 192 threads and 256 blocks does up to 63GB/s = 10.5GFlops 
 * This is a little lower than CUBLAS AXPY, so there might be some
 * little room for improvement
 * 
 */
 
#include "../kernels_config.h"

#ifndef _BX_TX
	#define _BX_TX
	#define tx          threadIdx.x 
	#define bx          blockIdx.x
#endif
#define X_OFFSET_SAYPX    0
#define Y_OFFSET_SAYPX    CUKR_SAYPX_THREADS

extern __shared__ double shared[];

__global__ void kernel_daypx(const double *d_x, 
                                   double *d_y, 
							 const double  alpha, 
                             const int     n)
{	
	int i;
				
	/* Alloc shared memory */
	double  *buffer_x = &shared[X_OFFSET_SAYPX]; 
	double  *buffer_y = &shared[Y_OFFSET_SAYPX]; 
	
	/* Starting point for this block */
	int ctaStart = bx * CUKR_SAYPX_THREADS; 
	/* Total no. of threads in the kernel */
	int totalThreads = CUKR_SAYPX_CTAS * CUKR_SAYPX_THREADS;
	
	/* Read the data (full lines) */
	for (i = ctaStart + tx; i < n; i += totalThreads)
	{  
		/* Prefetch x and y */
		buffer_x[tx] = d_x[i];
		buffer_y[tx] = d_y[i];
		
		/* Multiply and write */ 	 
		d_y[i] = buffer_x[tx] + alpha * buffer_y[tx];
	}  
	 
}
