/**
 * @file blas/gpu/cuda/kernel_dpmv_ell.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * SPMV in ELL format
 * Follows NVIDIA's implementation.
 * 
 + Both alpha and beta are considered. 
 */
 
#include "../kernels_textures.h"
#include "../kernels_utils.h"

#ifndef _BX_TX
	#define _BX_TX
	#define tx          threadIdx.x 
	#define bx          blockIdx.x
#endif

 
__global__ void kernel_dspmv_ell( const int     rows,      
						  	      const double  alpha,
						  	      const int     ell_nz_row,
						  	      const int     ell_stride,
						  	      const int    *ell_idx, 
								  const double *ell_val,
								  const double  beta,
						                double *d_y)
{
	const int row = large_grid_thread_id();

    if(row >= rows){ return; }

    double sum = 0;
    if (beta) sum = beta*d_y[row];
	
	ell_idx += row;
    ell_val += row;
	
	for(int n = 0; n < ell_nz_row; n++){
        const double A_ij = *ell_val;

        if (A_ij != 0){
            const int col = *ell_idx - 1;
            int2 x = tex1Dfetch(TEX_X2_I, col);
            sum += A_ij * __hiloint2double(x.y, x.x);
        }

        ell_idx += ell_stride;
        ell_val += ell_stride;
    }

    d_y[row] = sum;
}
