/**
 * @file blas/gpu/cuda/kernels_config.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Contains configuration stuff for the
 * CUDA kernels (e.g. threads, blocks, etc).
 * 
 */

#ifndef KERNELS_CONFIG_EMULATION_H_
#define KERNELS_CONFIG_EMULATION_H_

// ****************************  SETTINGS  ***************************** //

/** Set to 1 to use Kahan summation
 * in the SpMV kernels; 
 * Set to 0 to disable */
#define USE_KAHAN_IN_SPMV 0

// ****************************  FOR SINGLE  ***************************** //


/**
 * Number of thread blocks to be used
 * for the AYPX kernel 
 */
#define CUKR_SAYPX_CTAS    1
/** 
 * Number of threads per block to be
 * used for the AYPX kernel 
 */
#define CUKR_SAYPX_THREADS 1

// -------------------------------------------------------------- //

/** 
 * Number of thread blocks to be used
 * for the SPMV_CSR 
 */
#define CUKR_SSPMV_CSR_CTAS    1
/** 
 * Number of threads per block to be
 * used for the SPMV_CSR kernel 
 */
#define CUKR_SSPMV_CSR_THREADS 1

// -------------------------------------------------------------- //

/** 
 * Number of thread blocks to be used
 * for the SPMV_CSR 
 */
#define CUKR_SSPMV_CSR4_CTAS    1
/** 
 * Number of threads per block to be
 * used for the SPMV_CSR kernel 
 */
#define CUKR_SSPMV_CSR4_THREADS 1


// ****************************  FOR DOUBLE  ***************************** //


/**
 * Number of thread blocks to be used
 * for the AYPX kernel 
 */
#define CUKR_DAYPX_CTAS    1
/** 
 * Number of threads per block to be
 * used for the AYPX kernel 
 */
#define CUKR_DAYPX_THREADS 1


// -------------------------------------------------------------- //

/** 
 * Number of thread blocks to be used
 * for the SPMV_CSR 
 */
#define CUKR_DSPMV_CSR_CTAS    1
/** 
 * Number of threads per block to be
 * used for the SPMV_CSR kernel 
 */
#define CUKR_DSPMV_CSR_THREADS 1

// -------------------------------------------------------------- //

/** 
 * Number of thread blocks to be used
 * for the SPMV_CSR 
 */
#define CUKR_DSPMV_CSR4_CTAS    1
/** 
 * Number of threads per block to be
 * used for the SPMV_CSR kernel 
 */
#define CUKR_DSPMV_CSR4_THREADS 1



// *************************  FOR SINGLE-DOUBLE  ************************** //

/**
 * Number of thread blocks to be used
 * for the double-single DOT kernel 
 */
#define CUKR_QDOT_CTAS    1
/** 
 * Number of threads per block to be
 * used for the double-single DOT kernel 
 * IT MUST BE A POWER OF 2 !!! 
 */ 
#define CUKR_QDOT_THREADS 1 

// -------------------------------------------------------------- //

/**
 * Number of thread blocks to be used
 * for the double-single AXPY kernel 
 */
#define CUKR_QAXPY_CTAS    1
/** 
 * Number of threads per block to be
 * used for the double-single AXPY kernel 
 */
#define CUKR_QAXPY_THREADS 1


// -------------------------------------------------------------- //

/**
 * Number of thread blocks to be used
 * for the double-single AYPX kernel 
 */
#define CUKR_QAYPX_CTAS    1
/** 
 * Number of threads per block to be
 * used for the double-single AYPX kernel 
 */
#define CUKR_QAYPX_THREADS 1

// -------------------------------------------------------------- //

/**
 * Number of thread blocks to be used
 * for the double-single SCAL kernel 
 */
#define CUKR_QSCAL_CTAS    1
/** 
 * Number of threads per block to be
 * used for the double-single SCAL kernel 
 */ 
#define CUKR_QSCAL_THREADS 1

// -------------------------------------------------------------- //

/**
 * Number of thread blocks to be used
 * for the double-single CSR kernel,
 * where A = 1 (requires 18 regs) 
 */
#define CUKR_QSPMV_CSR_A1_CTAS    1 
/** 
 * Number of threads per block to be used
 * for the double-single CSR kernel,
 * where A = 1 (requires 18 regs) 
 */
#define CUKR_QSPMV_CSR_A1_THREADS 1

/**
 * Number of thread blocks to be used
 * for the double-single CSR kernel,
 * where A != 1 (requires 20 regs) 
 */
#define CUKR_QSPMV_CSR_CTAS    1
/** 
 * Number of threads per block to be used
 * for the double-single CSR kernel,
 * where A != 1 (requires 20 regs) 
 */
#define CUKR_QSPMV_CSR_THREADS 1


// -------------------------------------------------------------- //

/**
 * Number of thread blocks to be used
 * for the double-single SCAL kernel 
 */
#define CUKR_QSPMV_CSR4_CTAS    1
/** 
 * Number of threads per block to be
 * used for the double-single SCAL kernel 
 */
#define CUKR_QSPMV_CSR4_THREADS 1



#endif /*KERNELS_CONFIG_EMULATION_H_*/
