/**
 * @file blas/gpu/cuda/kernels_textures.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Contains texture declarations for spmv
 * kernels. It should therefore be included
 * in each SpMV-type kernel
 * 
 */

#ifndef KERNELS_TEXTURES_H_
#define KERNELS_TEXTURES_H_

/* Grid generation (from NVIDIA SPMV lib) */
#define large_grid_thread_id(void) ((__umul24(blockDim.x,blockIdx.x + __umul24(blockIdx.y,gridDim.x)) + threadIdx.x))
#define small_grid_thread_id(void) ((__umul24(blockDim.x, blockIdx.x) + threadIdx.x))

/** Texture for row indices */
texture<int,    1, cudaReadModeElementType> texPtr; 
/** Texture for column indices */
texture<int,    1, cudaReadModeElementType> texIdx;
/** Texture for column indices, for reading with int4 */
texture<int4,   1, cudaReadModeElementType> texIdx4;

/** Texture for values */
texture<float,  1, cudaReadModeElementType> texVal;
/** Texture for values, for reading with float2
 *  and then cast to double  */
texture<float2, 1, cudaReadModeElementType> texVal2;
texture<int2  , 1, cudaReadModeElementType> texVal2_i;
/** Texture for values, for reading with float4  */
texture<float4, 1, cudaReadModeElementType> texVal4;
texture<int4  , 1, cudaReadModeElementType> texVal4_i;
/** Texture for values (tail) */
texture<float,  1, cudaReadModeElementType> texVal_t;
/** Texture for values (tail), for reading with float4 */
texture<float4, 1, cudaReadModeElementType> texVal4_t;
/** Texture for x */
texture<float,  1, cudaReadModeElementType> texX;
/** Texture for x (tail) */
texture<float, 1, cudaReadModeElementType> texX_t;
/** Texture for x when used in double */
texture<float2, 1, cudaReadModeElementType> texX2;
texture<int2  , 1, cudaReadModeElementType> texX2_i;


#define TEX_PTR    texPtr
#define TEX_IDX    texIdx
#define TEX_IDX4   texIdx4
#define TEX_VAL    texVal
#define TEX_VAL2   texVal2
#define TEX_VAL2_I texVal2_i
#define TEX_VAL4   texVal4
#define TEX_VAL4_I texVal4_i
#define TEX_VAL_T  texVal_t
#define TEX_VAL4_T texVal4_t
#define TEX_X      texX
#define TEX_X_T    texX_t
#define TEX_X2     texX2
#define TEX_X2_I   texX2_i

#endif /*KERNELS_TEXTURES_H_*/
