/**
 * @file blas/gpu/cuda/kernels_config.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Contains configuration stuff for the
 * CUDA kernels (e.g. threads, blocks, etc).
 * 
 */ 

#pragma once

/* Grid generation (from NVIDIA SPMV lib) */
#define large_grid_thread_id(void) ((__umul24(blockDim.x,blockIdx.x + __umul24(blockIdx.y,gridDim.x)) + threadIdx.x)) 
#define small_grid_thread_id(void) ((__umul24(blockDim.x, blockIdx.x) + threadIdx.x))
 
/* int2 to double, from NVIDIA sm_13_double_functions.h" */
__device__ double __hiloint2double( int a, int b )
{
  volatile union {
    double     d;
    signed int i[2];
  } cvt;
  cvt.i[0] = b;
  cvt.i[1] = a;
  return cvt.d;
} 
