/**
 * @file blas/gpu/cuda/kernel_spmv_ell.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * SPMV in ELL format
 * Follows NVIDIA's implementation.
 * 
 + Both alpha and beta are considered. 
 */
 
#include "../kernels_textures.h"
#include "ds_ops.h"

#ifndef _BX_TX
	#define _BX_TX
	#define tx          threadIdx.x 
	#define bx          blockIdx.x
#endif

 
__global__ void kernel_qspmv_ell( const int    rows,      
						  	            float  alpha0,           float alpha1,
						  	      const int    ell_nz_row,
						  	      const int    ell_stride,
						  	      const int   *ell_idx, 
								        float *ell_valh,         float* ell_valt,
								        float  beta0,            float  beta1,
						                float *d_yh,             float* d_yt)
{
	const int row = large_grid_thread_id();

    if(row >= rows){ return; }

    float sum0 = 0, sum1 = 0;
    if (beta0 || beta1) dsmul(sum0, sum1, beta0, beta1, d_yh[row], d_yt[row]);
	
	ell_idx += row;
    ell_valh += row;
    ell_valt += row;
	
	for(int n = 0; n < ell_nz_row; n++){
        const float A_ij_h = *ell_valh;
        const float A_ij_t = *ell_valt;

        if (A_ij_h != 0){
     		float c0, c1;
            const int col = *ell_idx - 1;
            dsmul(c0, c1, A_ij_h, A_ij_t, tex1Dfetch(TEX_X, col), tex1Dfetch(TEX_X_T, col)); 
            dsadd(sum0, sum1, sum0, sum1, c0, c1);
        }

        ell_idx += ell_stride;
        ell_valh += ell_stride;
        ell_valt += ell_stride;
    }

    d_yh[row] = sum0;
    d_yt[row] = sum1;
}
