/**
 * @file blas/gpu/cuda/kernel_spmv_ell.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * SPMV in ELL format
 * Follows NVIDIA's implementation.
 * 
 + Both alpha and beta are considered. 
 */
 
#include "../kernels_textures.h"
#include "../kernels_utils.h"

#ifndef _BX_TX
	#define _BX_TX
	#define tx          threadIdx.x 
	#define bx          blockIdx.x
#endif

 
__global__ void kernel_sspmv_ell( const int    rows,      
						  	      const float  alpha,
						  	      const int    ell_nz_row,
						  	      const int    ell_stride,
						  	      const int   *ell_idx, 
								  const float *ell_val,
								  const float  beta,
						                float *d_y)
{
	const int row = large_grid_thread_id();

    if(row >= rows){ return; }
	
    float sum = 0;
    if (beta) sum = beta*d_y[row];
	
	ell_idx += row;
    ell_val += row;
	
	for(int n = 0; n < ell_nz_row; n++){
        const float A_ij = *ell_val;

        if (A_ij != 0){
            const int col = *ell_idx - 1;
            sum += A_ij * tex1Dfetch(TEX_X, col);
        }

        ell_idx += ell_stride;
        ell_val += ell_stride;
    }

    d_y[row] = sum;
}
