/*
 *  kernel_sdot.cl
 *  
 *
 *
 */
 
 #pragma OPENCL EXTENSION cl_khr_fp64 : enable

__kernel void kernel_ddot(
		const int n,    
		__global const double* x,
		__global const double* y,
		__global double* res_m,
		__local double* partial_sum
)
{
	double sum = 0.0;
					
	/* Starting point for this block */
	int ctaStart = get_group_id(0) * get_local_size(0);
	 
	/* Total no. of threads in the kernel */
	int totalThreads = get_global_size(0);
	
	/* Get current thread */
	int tx = get_local_id(0); 
	int bx = get_group_id(0); 
	
	/* Read the sum data */
	for (int i = ctaStart + tx; i < n; i += totalThreads) {
		sum += x[i]*y[i];
	}
	partial_sum[tx] = sum;
	
	/* Reduce data for the block */
	for (int i = get_local_size(0) >> 1; i > 0; i >>= 1)
	{
		barrier(CLK_LOCAL_MEM_FENCE); 
		if (tx < i) {
			partial_sum[tx] += partial_sum[tx + i];
		}	
	}

	if (tx == 0) {
       	res_m[bx] = partial_sum[tx];
    }
}