/*
 *  kernel_sspmv_csr4_b0.cl
 *  
 *
 * csr port
 *
 */
#pragma OPENCL EXTENSION cl_khr_fp64 : enable
 
__kernel void kernel_dspmv_csr4_b0(
		const int rows,
		__global const int* d_ptr,
		__global const int4* d_idx,
		__global const double4* d_val,
		__global const double* d_x,
		__global double* d_y,
		const double alpha
)
{
	/* Starting point for this block */
	int ctaStart = get_group_id(0) * get_local_size(0);
	 
	/* Total no. of threads in the kernel */
	int totalThreads = get_global_size(0);
	
	/* Get current thread */
	int tx = get_local_id(0);

	/* Read the data*/
	for (int i = ctaStart + tx; i < rows; i += totalThreads)
	{ 	
		/* Read the beginning and end of the row 
		 * which will be processed by this thread */
		int iRowBeg = d_ptr[i] - 1;
		int iRowEnd = d_ptr[i+1] - 1;
		
		/* Read and sum for the column vectors */
		double sum = 0.0;
		for (int j = iRowBeg / 4; j < iRowEnd / 4; j++) {
			double4 val, x;
			int4 idx;
			
			idx = d_idx[j];
			val = d_val[j];
			
			idx -= 1;
			
			x.x = d_x[idx.x];
			x.y = d_x[idx.y];
			x.z = d_x[idx.z];
			x.w = d_x[idx.w];
			
			sum += dot(x, val);
		}

		/* Write the result to global memory */
		d_y[i] = alpha * sum;		 	
	}
}