/*
 *  kernel_sspmv_ell.cl
 *  
 *
 * ell port
 *
 */
 
#pragma OPENCL EXTENSION cl_khr_fp64 : enable

#define large_grid_thread_id(void) (((uint)mul24((uint)get_local_size(0),(uint)get_group_id(0) + (uint)mul24((uint)get_group_id(1),(uint)get_num_groups(0))) + (uint)get_local_id(0)))

__kernel void kernel_dspmv_ell(
		const int rows,      
		const double alpha,
		const int ell_nz_row,
		const int ell_stride,
		__global const int *ell_idx, 
		__global const double *ell_val,
		const double beta,
		__global double *d_y,
		__global double *d_x
)
{
	const int row = large_grid_thread_id();
	
	if(row >= rows){
		return;
	}
	
	double sum = 0;
	if (beta)
		sum = beta * d_y[row];
	ell_idx += row;
	ell_val += row;
	for(int n = 0; n < ell_nz_row; n++){
		const double A_ij = *ell_val;
		
		if(A_ij != 0){
			const int col = *ell_idx - 1;
			sum += A_ij * d_x[col]; // this last d_x -> can be replaced by image access..
		}
		
		ell_idx += ell_stride;
		ell_val += ell_stride;
	}
	d_y[row] = sum;
}