/*
 *  kernels_single.c
 *  
 *
 *  Created by Olav Aanes Fagerlund.
 *
 */

#define LOGGER_NAME "CUKr.blas.impl.gpu.opencl.kernels_double"
#include "kernels_double.h"

#define AUTO_LOCAL 0
#define ORIGINAL_VALUES 0

#if ORIGINAL_VALUES
// Work-group sizes and the number of those equal to the CUDA version
#include "kernels_config_orig.h"
#else
// Work-group sizes and the number of those giving better OpenCL performance
#include "kernels_config.h"
#endif

double opencl_kernel_ddot(int n, cl_mem* x, cl_mem* y)
{	
	size_t sizes[5];
	void *values[5];
	
	double res = 0.0;
	double* res_arr = malloc(CUKR_DDOT_CTAS * sizeof(double));
	cl_mem res_buf = clCreateBuffer(ComputeContext, CL_MEM_WRITE_ONLY, CUKR_DDOT_CTAS * sizeof(cl_double), /*(void*)res*/ NULL, &err);
	if (err != CL_SUCCESS) {
		printf("Setting up res_buf for sdot failed!\n");
	}
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(cl_mem);
	values[1] = (void *)x;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)y;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)&res_buf;
	sizes[4] =  CUKR_DDOT_THREADS * sizeof(cl_double);
	values[4] = NULL;
	
	int x_inc;
	for (x_inc = 0; x_inc<5; x_inc++) {
		err = clSetKernelArg(ComputeKernel_ddot, x_inc, sizes[x_inc], values[x_inc]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d arg sdot\n", x_inc);
			//return -1;
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", x_inc);
			//return -1;
		}
	}
	
	size_t global[1];
	size_t local[1];
	
	local[0]  = CUKR_DDOT_THREADS;
	global[0] = CUKR_DDOT_CTAS * local[0];
	
	/* Enqueue kernel for execution */
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_ddot, 1, NULL, global, local, 0, NULL, &ddot_event);
	if (err != CL_SUCCESS)
	 {
		 printf("clExecuteKernel failed %d\n", n);
	 }
	
	// Synchronize for timing
	err = clWaitForEvents(1, &ddot_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
	
	// Get result
	err = clEnqueueReadBuffer(ComputeCommands, res_buf, CL_TRUE, 0, CUKR_DDOT_CTAS * sizeof(cl_double), (void*)res_arr, 0, NULL, NULL);
	if (err == CL_INVALID_COMMAND_QUEUE) {
		printf("YES!!!\n");
	}
	else if (err != CL_SUCCESS) {
		printf("readback of res_buf in sdot failed! err-code: %d\n", err);
	}
	
	err = clReleaseMemObject(res_buf);
	if (err != CL_SUCCESS) {
		printf("release of cl memobject res_buf in sdot failed!\n");
	}
	
	int inc;
	for (inc = 0; inc < CUKR_DDOT_CTAS; inc++) {
		res += res_arr[inc];
	}
	
	// Free the array
	free(res_arr);
	res_arr = NULL;
	
	return res;	
}

void opencl_kernel_daxpy(int n, double a, cl_mem* cl_d_x, cl_mem* cl_d_y)
{
	size_t sizes[6];
	void *values[6];
	
	size_t global[1];
	size_t local[1];

	local[0] = CUKR_DAXPY_THREADS;
	global[0] = local[0] * CUKR_DAXPY_CTAS;

	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(double);
	values[1] = (void *)&a;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)cl_d_x;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)cl_d_y;
	
	sizes[4] = CUKR_DAXPY_THREADS * sizeof(cl_double);
	values[4] = NULL;
	sizes[5] = CUKR_DAXPY_THREADS * sizeof(cl_double);
	values[5] = NULL;
	
	/* Kernel invocation */
	int x;
	for (x = 0; x<6; x++) {
		err = clSetKernelArg(ComputeKernel_daxpy, x, sizes[x], values[x]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d failed\n", x);
			//return -1;
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", x);
			//return -1;
		}
	}
	
	/* Enqueue kernel for execution */
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_daxpy, 1, NULL, global, local, 0, NULL, &daxpy_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel failed %d\n",n);
	}
	
	// Synchronize for timing
	err = clWaitForEvents(1, &daxpy_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
	
}

void opencl_kernel_daypx(int n, double a, cl_mem* cl_d_x, cl_mem* cl_d_y)
{
	size_t sizes[6];
	void *values[6];
	
	size_t global[1];
	size_t local[1];
	
	local[0] = CUKR_DAYPX_THREADS;
	global[0] = local[0] * CUKR_DAYPX_CTAS;
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(double);
	values[1] = (void *)&a;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)cl_d_x;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)cl_d_y;
	
	sizes[4] = CUKR_DAYPX_THREADS * sizeof(cl_double);
	values[4] = NULL;
	sizes[5] = CUKR_DAYPX_THREADS * sizeof(cl_double);
	values[5] = NULL;
	
	/* Kernel invocation */
	int x;
	for (x = 0; x<6; x++) {
		err = clSetKernelArg(ComputeKernel_daypx, x, sizes[x], values[x]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d failed\n", x);
			//return -1;
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", x);
			//return -1;
		}
	}

	/* Enqueue kernel for execution */
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_daypx, 1, NULL, global, local, 0, NULL, &daypx_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel failed %d\n",n);
	}

	// Synchronize for timing
	err = clWaitForEvents(1, &daypx_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
}

void opencl_kernel_dscal(int n, double a, cl_mem* x)
{
	size_t sizes[4];
	void *values[4];
	
	size_t global[1];
	size_t local[1];

	local[0] = CUKR_DSCAL_THREADS;
	global[0] = local[0] * CUKR_DSCAL_CTAS;
	
	if (x == NULL) {
		printf("NULL POINTER x!\n");
	}
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(double);
	values[1] = (void *)&a;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)x;
	sizes[3] = CUKR_DSCAL_THREADS * sizeof(cl_double);
	values[3] = NULL;
	
	/* Kernel invocation */
	int i;
	for (i = 0; i<4; i++) {
		err = clSetKernelArg(ComputeKernel_dscal, i, sizes[i], values[i]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d failed\n", i);
			//return -1;
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", i);
			//return -1;
		}
	}
	
	/* Enqueue kernel for execution */
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_dscal, 1, NULL, global, local, 0, NULL, &dscal_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel sscal failed %d\n",n);
	}
	
	// Synchronize for timing
	err = clWaitForEvents(1, &dscal_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
}


void opencl_dcopy(int n, cl_mem* cl_d_x, int incx, cl_mem* cl_d_y, int incy)
{
	err = clEnqueueCopyBuffer(ComputeCommands, *cl_d_x, *cl_d_y, 0, 0, n * sizeof(cl_double), 0, NULL, &dcopy_event);
	if (err != CL_SUCCESS)
	{
		printf("clEnqueueCopyBuffer failed %d\n",n);
	}
	
	// Synchronize for timing
	err = clWaitForEvents(1, &dcopy_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
}

void opencl_dspmv_csr(int rows, int cols, int nz, double alpha, cl_mem* d_ptr, 
					  cl_mem* d_idx, cl_mem* d_val, cl_mem* d_x, double beta, cl_mem* d_y)
{	
	size_t sizes[7];
	void *values[7];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&rows;
	sizes[1] = sizeof(cl_mem);
	values[1] = (void *)d_ptr;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)d_idx;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)d_val;
	sizes[4] = sizeof(cl_mem);
	values[4] = (void *)d_x;
	sizes[5] = sizeof(cl_mem);
	values[5] = (void *)d_y;
	sizes[6] = sizeof(double);
	values[6] = (void *)&alpha;
	
	size_t global[1];
	size_t local[1];
	
	int inc;
	
	/* If beta != 1, do a scaling first */
	if (beta != 1 && beta)
		opencl_kernel_dscal(rows, beta, d_y);

	if (beta) {
		
		local[0] = CUKR_DSPMV_CSR_THREADS; //512;
		global[0] = CUKR_DSPMV_CSR_CTAS * local[0];
		
		// Set kernel args
		for (inc = 0; inc < 7; inc++) {
			err = clSetKernelArg(ComputeKernel_dspmv_csr, inc, sizes[inc], values[inc]);
			if (err == CL_INVALID_KERNEL)
			{
				printf("Failed %d failed\n", inc);
				//return -1;
			}
			if (err != CL_SUCCESS)
			{
				printf("clSetKernelArg %d failed\n", inc);
				//return -1;
			}
		}
		err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_dspmv_csr, 1, NULL, global, local, 0, NULL, &dspmv_csr_event);
		if (err != CL_SUCCESS)
		{
			printf("clExecuteKernel sspmv_csr failed %d\n", nz);
		}
		// Synchronize for timing
		err = clWaitForEvents(1, &dspmv_csr_event);
		if (err != CL_SUCCESS) {
			printf("clWaitForEvents failed!\n");
		}
	}

	else {
		// Set workgroup sizes
		local[0] = CUKR_DSPMV_CSR_THREADS; //512;
		global[0] = CUKR_DSPMV_CSR_CTAS * local[0];
		
		// Set kernel args
		for (inc = 0; inc < 7; inc++) {
			err = clSetKernelArg(ComputeKernel_dspmv_csr_b0, inc, sizes[inc], values[inc]);
			if (err == CL_INVALID_KERNEL)
			{
				printf("Failed %d failed\n", inc);
				//return -1;
			}
			if (err != CL_SUCCESS)
			{
				printf("clSetKernelArg %d failed\n", inc);
				//return -1;
			}
		}
		err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_dspmv_csr_b0, 1, NULL, global, local, 0, NULL, &dspmv_csr_b0_event);
		if (err != CL_SUCCESS)
		{
			printf("clExecuteKernel sspmv_csr_b0 failed %d\n", nz);
		}
		// Synchronize for timing
		err = clWaitForEvents(1, &dspmv_csr_b0_event);
		if (err != CL_SUCCESS) {
			printf("clWaitForEvents failed!\n");
		}
	}
}

void opencl_dspmv_csr4(int rows, int cols, int nz, double alpha, cl_mem* d_ptr, 
					   cl_mem* d_idx, cl_mem* d_val, cl_mem* d_x, double beta, cl_mem* d_y)
{	
	size_t sizes[7];
	void *values[7];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&rows;
	sizes[1] = sizeof(cl_mem);
	values[1] = (void *)d_ptr;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)d_idx;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)d_val;
	sizes[4] = sizeof(cl_mem);
	values[4] = (void *)d_x;
	sizes[5] = sizeof(cl_mem);
	values[5] = (void *)d_y;
	sizes[6] = sizeof(double);
	values[6] = (void *)&alpha;
	
	size_t global[1];
	size_t local[1];
	
	int inc;
	
	if (beta != 1 && beta)
		opencl_kernel_sscal(rows, beta, d_y);
		
	 	/* If alpha = 1, no need to consider */
	if (beta) {
		// Set workgroup sizes
		local[0] = CUKR_DSPMV_CSR4_THREADS; //512;
		global[0] = CUKR_DSPMV_CSR4_CTAS * local[0];
			
		// Set kernel args
		for (inc = 0; inc < 7; inc++) {
			err = clSetKernelArg(ComputeKernel_dspmv_csr4, inc, sizes[inc], values[inc]);
			if (err == CL_INVALID_KERNEL)
			{
				printf("Failed %d failed\n", inc);
				//return -1;
			}
			if (err != CL_SUCCESS)
			{
				printf("clSetKernelArg %d failed\n", inc);
				//return -1;
			}
		}
		err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_dspmv_csr4, 1, NULL, global, local, 0, NULL, &dspmv_csr4_event);
		if (err != CL_SUCCESS)
		{
			printf("clExecuteKernel sspmv_csr4 failed %d\n", nz);
		}
		// Synchronize for timing
		err = clWaitForEvents(1, &dspmv_csr4_event);
		if (err != CL_SUCCESS) {
			printf("clWaitForEvents failed!\n");
		}
	}

	else {
		// Set workgroup sizes
		local[0] = CUKR_DSPMV_CSR4_THREADS; //512;
		global[0] = CUKR_DSPMV_CSR4_CTAS * local[0];
			
		// Set kernel args
		for (inc = 0; inc < 7; inc++) {
			err = clSetKernelArg(ComputeKernel_dspmv_csr4_b0, inc, sizes[inc], values[inc]);
			if (err == CL_INVALID_KERNEL)
			{
				printf("Failed %d failed\n", inc);
				//return -1;
			}
			if (err != CL_SUCCESS)
			{
				printf("clSetKernelArg %d failed\n", inc);
				//return -1;
			}
		}
		err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_dspmv_csr4_b0, 1, NULL, global, local, 0, NULL, &dspmv_csr4_b0_event);
		if (err != CL_SUCCESS)
		{
			printf("clExecuteKernel sspmv_csr4_b0 failed %d\n", nz);
		}
		// Synchronize for timing
		err = clWaitForEvents(1, &dspmv_csr4_b0_event);
		if (err != CL_SUCCESS) {
			printf("clWaitForEvents failed!\n");
		}
	}
}

void opencl_dspmv_hyb(int rows, int cols, int nz, double alpha, int ell_nz_row, int ell_stride, int csr_nz, 
                    cl_mem* d_ell_idx, cl_mem* d_ell_val, cl_mem* d_csr_ptr, cl_mem* d_csr_idx, cl_mem* d_csr_val,  
                    cl_mem* d_x, double beta, cl_mem* d_y)    
{		
#define DIVIDE_INTO(x, y) (((x) + (y) - 1)/(y))
	
	const unsigned int BLOCK_SIZE_ELL = 256;
	unsigned int num_blocks = DIVIDE_INTO(rows, BLOCK_SIZE_ELL);
	
	// Prepare data for the ell kernel
	size_t sizes[9];
	void *values[9];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&rows;
	sizes[1] = sizeof(double);
	values[1] = (void *)&alpha;
	sizes[2] = sizeof(int);
	values[2] = (void *)&ell_nz_row;
	sizes[3] = sizeof(int);
	values[3] = (void *)&ell_stride;
	sizes[4] = sizeof(cl_mem);
	values[4] = (void *)d_ell_idx;
	sizes[5] = sizeof(cl_mem);
	values[5] = (void *)d_ell_val;
	sizes[6] = sizeof(double);
	values[6] = (void *)&beta;
	sizes[7] = sizeof(cl_mem);
	values[7] = (void *)d_y;
	sizes[8] = sizeof(cl_mem);
	values[8] = (void *)d_x;
	
	size_t global[1];
	size_t local[1];
	
	local[0] = BLOCK_SIZE_ELL;
	global[0] = num_blocks * local[0];
	
	int inc;
	// Set kernel args
	for (inc = 0; inc < 9; inc++) {
		err = clSetKernelArg(ComputeKernel_dspmv_ell, inc, sizes[inc], values[inc]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d failed\n", inc);
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", inc);
		}
	}
	
	// Launch the kernel
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_dspmv_ell, 1, NULL, global, local, 0, NULL, &dspmv_hyb_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel dspmv_ell failed %d\n", nz);
	}
	
	local[0] = CUKR_DSPMV_CSR4_THREADS;
	global[0] = CUKR_DSPMV_CSR4_CTAS * local[0];
	
	if(csr_nz){
		// The rest in CSR4
			sizes[0] = sizeof(int);
			values[0] = (void *)&rows;
			sizes[1] = sizeof(cl_mem);
			values[1] = (void *)d_csr_ptr;
			sizes[2] = sizeof(cl_mem);
			values[2] = (void *)d_csr_idx;
			sizes[3] = sizeof(cl_mem);
			values[3] = (void *)d_csr_val;
			sizes[4] = sizeof(cl_mem);
			values[4] = (void *)d_x;
			sizes[5] = sizeof(cl_mem);
			values[5] = (void *)d_y;
			sizes[6] = sizeof(double);
			values[6] = (void *)&alpha;
			
			// Set kernel args
			for (inc = 0; inc < 7; inc++) {
				err = clSetKernelArg(ComputeKernel_dspmv_csr4, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_dspmv_csr4, 1, NULL, global, local, 0, NULL, &dspmv_hyb_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel dspmv_csr4 failed %d\n", nz);
			}
	}
	// Synchronize for timing
	err = clWaitForEvents(1, &dspmv_hyb_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
}

void opencl_dspmv_bcsr(int rows, int cols, int nz, int r, int c, double alpha, cl_mem* ptr, 
					   cl_mem* idx, cl_mem* val, cl_mem* x, double beta, 
					   cl_mem* y)
{
	printf("test opencl_dspmv_bcsr\n");
}

void opencl_dspmv_bcsr4(int rows, int cols, int nz, int r, int c, double alpha, cl_mem* ptr, 
						cl_mem* idx, cl_mem* val, cl_mem* x, double beta, 
						cl_mem* y)
{
	printf("test opencl_dspmv_bcsr4\n");
}
