/*
 *  kernels_single.c
 *  
 *
 *  Created by Olav Aanes Fagerlund.
 *
 */

#define LOGGER_NAME "CUKr.blas.impl.gpu.opencl.kernels_single"
#include "kernels_single.h"

#define AUTO_LOCAL 0
#define ORIGINAL_VALUES 0

#if ORIGINAL_VALUES
// Work-group sizes and the number of those equal to the CUDA version
#include "kernels_config_orig.h"
#else
// Work-group sizes and the number of those giving better OpenCL performance
#include "kernels_config.h"
#endif

float opencl_kernel_sdot(int n, cl_mem* x, cl_mem* y)
{

	int LOCAL_SIZE = CUKR_SDOT_THREADS;
	
	#if AUTO_LOCAL
	LOCAL_SIZE = AUTO_LOCAL_SIZE_SDOT;
	#endif

	size_t sizes[5];
	void *values[5];
	
	float res = 0;
	float* res_arr = malloc(CUKR_SDOT_CTAS * sizeof(float));
	cl_mem res_buf = clCreateBuffer(ComputeContext, CL_MEM_WRITE_ONLY, CUKR_SDOT_CTAS * sizeof(cl_float), /*(void*)res*/ NULL, &err);
	if (err != CL_SUCCESS) {
		printf("Setting up res_buf for sdot failed!\n");
	}
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(cl_mem);
	values[1] = (void *)x;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)y;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)&res_buf;
	sizes[4] =  LOCAL_SIZE * sizeof(cl_float);
	values[4] = NULL;
	
	int x_inc;
	for (x_inc = 0; x_inc<5; x_inc++) {
		err = clSetKernelArg(ComputeKernel_sdot, x_inc, sizes[x_inc], values[x_inc]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d arg sdot\n", x_inc);
			//return -1;
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", x_inc);
			//return -1;
		}
	}
	
	size_t global[1];
	size_t local[1];
	
	local[0]  = LOCAL_SIZE;
	global[0] = CUKR_SDOT_CTAS * local[0];
	
	/* Enqueue kernel for execution */
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sdot, 1, NULL, global, local, 0, NULL, &sdot_event);
	
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel failed %d\n", n);
	}

	// Synchronize for timing
	err = clWaitForEvents(1, &sdot_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
	
	// Get result
	err = clEnqueueReadBuffer(ComputeCommands, res_buf, CL_TRUE, 0, CUKR_SDOT_CTAS * sizeof(cl_float), (void*)res_arr, 0, NULL, NULL);
	if (err == CL_INVALID_COMMAND_QUEUE) {
		printf("YES!!!\n");
	}
	else if (err != CL_SUCCESS) {
		printf("readback of res_buf in sdot failed! err-code: %d\n", err);
	}
	
	err = clReleaseMemObject(res_buf);
	if (err != CL_SUCCESS) {
		printf("release of cl memobject res_buf in sdot failed!\n");
	}
	
	int inc;
	for (inc = 0; inc < CUKR_SDOT_CTAS; inc++) {
		res += res_arr[inc];
	}
	
	// free the array
	free(res_arr);
	res_arr = NULL;

	return res;
}

void opencl_kernel_saxpy(int n, float a, cl_mem* cl_d_x, cl_mem* cl_d_y)
{
	// Set workgroup sizes
	/*
	err = clGetKernelWorkGroupInfo(ComputeKernel_saxpy, ComputeDeviceId, CL_KERNEL_WORK_GROUP_SIZE, sizeof(size_t), &auto_local, NULL);
	if (err != CL_SUCCESS) {
		printf("clGetKernelWorkGroupInfo failed! : %d\n", err);
	}
	*/
	
	size_t sizes[6];
	void *values[6];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(float);
	values[1] = (void *)&a;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)cl_d_x;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)cl_d_y;
	
	sizes[4] = CUKR_SAXPY_THREADS * sizeof(cl_float);
	values[4] = NULL;
	sizes[5] = CUKR_SAXPY_THREADS * sizeof(cl_float);
	values[5] = NULL;
	
	/* Kernel invocation */
	int x;
	for (x = 0; x<6; x++) {
		err = clSetKernelArg(ComputeKernel_saxpy, x, sizes[x], values[x]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d failed\n", x);
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", x);
		}
	}
	
	size_t global[1];
	size_t local[1];
	
	local[0] = CUKR_SAXPY_THREADS;
	global[0] = CUKR_SAXPY_CTAS * local[0];
	
	/* Enqueue kernel for execution */
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_saxpy, 1, NULL, global, local, 0, NULL, &saxpy_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel failed %d\n",n);
	}
	
	// Synchronize for timing
	err = clWaitForEvents(1, &saxpy_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
}

void opencl_kernel_saypx(int n, float a, cl_mem* cl_d_x, cl_mem* cl_d_y)
{
	// Set workgroup sizes
	/*
	err = clGetKernelWorkGroupInfo(ComputeKernel_saypx, ComputeDeviceId, CL_KERNEL_WORK_GROUP_SIZE, sizeof(size_t), &auto_local, NULL);
	if (err != CL_SUCCESS) {
		printf("clGetKernelWorkGroupInfo failed! : %d\n", err);
	}
	*/
	
	size_t sizes[6];
	void *values[6];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(float);
	values[1] = (void *)&a;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)cl_d_x;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)cl_d_y;
	
	sizes[4] = CUKR_SAYPX_THREADS * sizeof(cl_float);
	values[4] = NULL;
	sizes[5] = CUKR_SAYPX_THREADS * sizeof(cl_float);
	values[5] = NULL;

	// Set kernel args
	int x;
	for (x = 0; x<6; x++) {
		err = clSetKernelArg(ComputeKernel_saypx, x, sizes[x], values[x]);
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", x);
			//return -1;
		}
	}
	//}

	size_t global[1];
	size_t local[1];
	
	local[0] = CUKR_SAYPX_THREADS;
	global[0] = CUKR_SAYPX_CTAS * local[0];

	// Enqueue kernel for execution
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_saypx, 1, NULL, global, local, 0, NULL, &saypx_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel failed %d\n",n);
	}
	
	err = clWaitForEvents(1, &saypx_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}

}

void opencl_kernel_sscal(int n, float a, cl_mem* x)
{	
	size_t sizes[4];
	void *values[4];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&n;
	sizes[1] = sizeof(float);
	values[1] = (void *)&a;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)x;
	sizes[3] = CUKR_SSCAL_THREADS * sizeof(cl_float);
	values[3] = NULL;
	
	/* Kernel invocation */
	int i;
	for (i = 0; i<4; i++) {
		err = clSetKernelArg(ComputeKernel_sscal, i, sizes[i], values[i]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d failed\n", i);
			//return -1;
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", i);
			//return -1;
		}
	}
	
	size_t global[1];
	size_t local[1];
	
	local[0] = CUKR_SSCAL_THREADS;
	global[0] = CUKR_SSCAL_CTAS * local[0];
	
	/* Enqueue kernel for execution */
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sscal, 1, NULL, global, local, 0, NULL, &sscal_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel sscal failed %d\n",n);
	}
	
	// Synchronize for timing
	err = clWaitForEvents(1, &sscal_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
	
}

void opencl_scopy(int n, cl_mem* cl_d_x, int incx, cl_mem* cl_d_y, int incy)
{
	err = clEnqueueCopyBuffer(ComputeCommands, *cl_d_x, *cl_d_y, 0, 0, n * sizeof(cl_float), 0, NULL, &scopy_event);
	if (err != CL_SUCCESS)
	{
		printf("clEnqueueCopyBuffer failed %d\n", n);
	}
	
	// Synchronize for timing
	err = clWaitForEvents(1, &scopy_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
}

void opencl_sspmv_csr(int rows, int cols, int nz, float alpha, cl_mem* d_ptr, 
					  cl_mem* d_idx, cl_mem* d_val, cl_mem* d_x, float beta, cl_mem* d_y)
{	
	size_t sizes[7];
	void *values[7];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&rows;
	sizes[1] = sizeof(cl_mem);
	values[1] = (void *)d_ptr;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)d_idx;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)d_val;
	sizes[4] = sizeof(cl_mem);
	values[4] = (void *)d_x;
	sizes[5] = sizeof(cl_mem);
	values[5] = (void *)d_y;
	sizes[6] = sizeof(float);
	values[6] = (void *)&alpha;
	
	size_t global[1];
	size_t local[1];
	
	int inc;
	
	/* If beta != 0 */
	if (beta != 0)
	{
	 	/* If beta != 1, do a scaling first */
	 	if (beta != 1)
	 		opencl_kernel_sscal(rows, beta, d_y);
		
	 	/* If alpha = 1, no need to consider */
   		if (alpha == 1) {
			// Set workgroup sizes
			local[0] = CUKR_SSPMV_CSR_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 6; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr_a1, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr_a1, 1, NULL, global, local, 0, NULL, &sspmv_csr_a1_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr_a1 failed %d\n", nz);
			}
			
			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr_a1_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
   		}
	 	/* Else, take the most general case */
	 	else {
			// Set workgroup sizes
			local[0] = CUKR_SSPMV_CSR_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 7; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr, 1, NULL, global, local, 0, NULL, &sspmv_csr_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr failed %d\n", nz);
			}
			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
	 	}
		
	}
	/* If beta = 0 */
	else {
	 	/* If alpha = 1 as well, no need to
	 	 * consider them both */
		if (alpha == 1) {
	 	 	// Set workgroup sizes
			local[0] = CUKR_SSPMV_CSR_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 6; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr_a1_b0, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr_a1_b0, 1, NULL, global, local, 0, NULL, &sspmv_csr_a1_b0_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr_a1_b0 failed %d\n", nz);
			}
			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr_a1_b0_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
		}
   	    /* If alpha != 1, have to consider it */
   	    else {
   	    	// Set workgroup sizes
			local[0] = CUKR_SSPMV_CSR_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 7; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr_b0, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr_b0, 1, NULL, global, local, 0, NULL, &sspmv_csr_b0_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr_b0 failed %d\n", nz);
			}
			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr_b0_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
   	    }
		
	}
}

void opencl_sspmv_csr4(int rows, int cols, int nz, float alpha, cl_mem* d_ptr, 
					  cl_mem* d_idx, cl_mem* d_val, cl_mem* d_x, float beta, cl_mem* d_y)
{	
	size_t sizes[7];
	void *values[7];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&rows;
	sizes[1] = sizeof(cl_mem);
	values[1] = (void *)d_ptr;
	sizes[2] = sizeof(cl_mem);
	values[2] = (void *)d_idx;
	sizes[3] = sizeof(cl_mem);
	values[3] = (void *)d_val;
	sizes[4] = sizeof(cl_mem);
	values[4] = (void *)d_x;
	sizes[5] = sizeof(cl_mem);
	values[5] = (void *)d_y;
	sizes[6] = sizeof(float);
	values[6] = (void *)&alpha;
	
	size_t global[1];
	size_t local[1];
	
	int inc;
	
	/* If beta != 0 */
	if (beta != 0)
	{
	 	/* If beta != 1, do a scaling first */
	 	if (beta != 1)
	 		opencl_kernel_sscal(rows, beta, d_y);
		
	 	/* If alpha = 1, no need to consider */
   		if (alpha == 1) {
   	   		// Set workgroup sizes
			local[0] = CUKR_SSPMV_CSR4_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR4_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 6; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr4_a1, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr4_a1, 1, NULL, global, local, 0, NULL, &sspmv_csr4_a1_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr4_a1 failed %d\n", nz);
			}
			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr4_a1_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
   		}
	 	/* Else, take the most general case */
	 	else {
	 		// Set workgroup sizes
			size_t auto_local;
			local[0] = CUKR_SSPMV_CSR4_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR4_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 7; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr4, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr4, 1, NULL, global, local, 0, NULL, &sspmv_csr4_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr4 failed %d\n", nz);
			}
			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr4_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
	 	}
		
	}
	/* If beta = 0 */
	else {
	 	/* If alpha = 1 as well, no need to
	 	 * consider them both */
		if (alpha == 1) {
	 	 	// Set workgroup sizes
			local[0] = CUKR_SSPMV_CSR4_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR4_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 6; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr4_a1_b0, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr4_a1_b0, 1, NULL, global, local, 0, NULL, &sspmv_csr4_a1_b0_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr4_a1_b0 failed %d\n", nz);
			}

			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr4_a1_b0_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
		}
   	    /* If alpha != 1, have to consider it */
   	    else {
   	    	// Set workgroup sizes
			local[0] = CUKR_SSPMV_CSR4_THREADS; //512;
			global[0] = CUKR_SSPMV_CSR4_CTAS * local[0];
			
			// Set kernel args
			for (inc = 0; inc < 7; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr4_b0, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
					//return -1;
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
					//return -1;
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr4_b0, 1, NULL, global, local, 0, NULL, &sspmv_csr4_b0_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr4_b0 failed %d\n", nz);
			}
			// Synchronize for timing
			err = clWaitForEvents(1, &sspmv_csr4_b0_event);
			if (err != CL_SUCCESS) {
				printf("clWaitForEvents failed!\n");
			}
   	    }
		
	}
}

/**
 * \brief Wrapper for SSPMV_HYB 
 * \param rows,cols Matrix size
 * \param nz Number of nonzeros
 * \param alpha Scale factor for Ax
 * \param ell_nz_row,ell_stride ELL dimensions 
 * \param csr_nz CSR dimension
 * \param ell_idx ELL column index vector
 * \param ell_val ELL value vector
 * \param csr_ptr CSR ptr vector
 * \param csr_idx CSR idx vector 
 * \param csr_val CSR value vector
 * \param x Vector being multiplied
 * \param beta Scale factor for y
 * \param y Result vector
 */  
void opencl_sspmv_hyb(int rows, int cols, int nz, float alpha, int ell_nz_row, int ell_stride, int csr_nz, 
                    cl_mem* d_ell_idx, cl_mem* d_ell_val, cl_mem* d_csr_ptr, cl_mem* d_csr_idx, cl_mem* d_csr_val,  
                    cl_mem* d_x, float beta, cl_mem* d_y)
{
	#define DIVIDE_INTO(x, y) (((x) + (y) - 1)/(y))

	const unsigned int BLOCK_SIZE_ELL = 256;
	unsigned int num_blocks = DIVIDE_INTO(rows, BLOCK_SIZE_ELL);
	
	// Prepare data for the ell kernel
	size_t sizes[9];
	void *values[9];
	
	sizes[0] = sizeof(int);
	values[0] = (void *)&rows;
	sizes[1] = sizeof(float);
	values[1] = (void *)&alpha;
	sizes[2] = sizeof(int);
	values[2] = (void *)&ell_nz_row;
	sizes[3] = sizeof(int);
	values[3] = (void *)&ell_stride;
	sizes[4] = sizeof(cl_mem);
	values[4] = (void *)d_ell_idx;
	sizes[5] = sizeof(cl_mem);
	values[5] = (void *)d_ell_val;
	sizes[6] = sizeof(float);
	values[6] = (void *)&beta;
	sizes[7] = sizeof(cl_mem);
	values[7] = (void *)d_y;
	sizes[8] = sizeof(cl_mem);
	values[8] = (void *)d_x;
	
	size_t global[1];
	size_t local[1];
	
	local[0] = BLOCK_SIZE_ELL;
	global[0] = num_blocks * local[0];
	
	int inc;
	// Set kernel args
	for (inc = 0; inc < 9; inc++) {
		err = clSetKernelArg(ComputeKernel_sspmv_ell, inc, sizes[inc], values[inc]);
		if (err == CL_INVALID_KERNEL)
		{
			printf("Failed %d failed\n", inc);
		}
		if (err != CL_SUCCESS)
		{
			printf("clSetKernelArg %d failed\n", inc);
		}
	}
	
	// Launch the kernel
	err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_ell, 1, NULL, global, local, 0, NULL, &sspmv_hyb_event);
	if (err != CL_SUCCESS)
	{
		printf("clExecuteKernel sspmv_ell failed %d\n", nz);
	}

	local[0] = CUKR_SSPMV_CSR4_THREADS;
	global[0] = CUKR_SSPMV_CSR4_CTAS * local[0];
	
	if(csr_nz){
		// The rest in CSR4
		if (alpha == 1){
			sizes[0] = sizeof(int);
			values[0] = (void *)&rows;
			sizes[1] = sizeof(cl_mem);
			values[1] = (void *)d_csr_ptr;
			sizes[2] = sizeof(cl_mem);
			values[2] = (void *)d_csr_idx;
			sizes[3] = sizeof(cl_mem);
			values[3] = (void *)d_csr_val;
			sizes[4] = sizeof(cl_mem);
			values[4] = (void *)d_x;
			sizes[5] = sizeof(cl_mem);
			values[5] = (void *)d_y;

			// Set kernel args
			for (inc = 0; inc < 6; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr4_a1, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr4_a1, 1, NULL, global, local, 0, NULL, &sspmv_hyb_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr4_a1 failed %d\n", nz);
			}
		}
		else {
			sizes[0] = sizeof(int);
			values[0] = (void *)&rows;
			sizes[1] = sizeof(cl_mem);
			values[1] = (void *)d_csr_ptr;
			sizes[2] = sizeof(cl_mem);
			values[2] = (void *)d_csr_idx;
			sizes[3] = sizeof(cl_mem);
			values[3] = (void *)d_csr_val;
			sizes[4] = sizeof(cl_mem);
			values[4] = (void *)d_x;
			sizes[5] = sizeof(cl_mem);
			values[5] = (void *)d_y;
			sizes[6] = sizeof(float);
			values[6] = (void *)&alpha;
			
			// Set kernel args
			for (inc = 0; inc < 7; inc++) {
				err = clSetKernelArg(ComputeKernel_sspmv_csr4, inc, sizes[inc], values[inc]);
				if (err == CL_INVALID_KERNEL)
				{
					printf("Failed %d failed\n", inc);
				}
				if (err != CL_SUCCESS)
				{
					printf("clSetKernelArg %d failed\n", inc);
				}
			}
			err = clEnqueueNDRangeKernel(ComputeCommands, ComputeKernel_sspmv_csr4, 1, NULL, global, local, 0, NULL, &sspmv_hyb_event);
			if (err != CL_SUCCESS)
			{
				printf("clExecuteKernel sspmv_csr4 failed %d\n", nz);
			}
		}
	}
	
	// Synchronize for timing
	err = clWaitForEvents(1, &sspmv_hyb_event);
	if (err != CL_SUCCESS) {
		printf("clWaitForEvents failed!\n");
	}
}

void opencl_sspmv_bcsr(int rows, int cols, int nz, int r, int c, float alpha, cl_mem* ptr, 
					   cl_mem* idx, cl_mem* val, cl_mem* x, float beta, 
					   cl_mem* y)
{
	printf("test opencl sspmv bcsr\n");
}

void opencl_sspmv_bcsr4(int rows, int cols, int nz, int r, int c, float alpha, cl_mem* ptr, 
					 cl_mem* idx, cl_mem* val, cl_mem* x, float beta, 
					 cl_mem* y)
{
	printf("test opencl sspmv bcsr4\n");
}
