/*
 *  kernel_saxpy.cl
 *  
 *
 *
 */

__kernel void kernel_saxpy(
		const int n,  
		const float a,  
		__global const float* x,
		__global float* y,
		__local float* buffer_x,
		__local float* buffer_y
)
{
	/* Starting point for this block */
	int ctaStart = get_group_id(0) * get_local_size(0);
	 
	/* Total no. of threads in the kernel */
	int totalThreads = get_global_size(0);
	
	/* Get current thread */
	int tx = get_local_id(0); 
	
	/* Read the data (full lines) */
	for (int i = ctaStart + tx; i < n; i += totalThreads)
	{  
		/* Prefetch x and y */
		buffer_x[tx] = x[i];
		buffer_y[tx] = y[i];

		/* Multiply and write */ 	 
		y[i] = buffer_y[tx] + a * buffer_x[tx];
		//y[i] = y[i] + a * x[i];
	}	
}