/*
 *  kernel_saxpy.cl
 *  
 *
 *
 */

__kernel void kernel_saxpy(
		const int n,  
		const float a,  
		__global const float4* x,
		__global float4* y
)
{
	int work_item_domain;
	int totalWorkItems = get_global_size(0);
	
	// get amount of overshooting float values (0, 1, 2, or 3)
	int restSingle = n % 4;
	
	// amount of full float4 values
	int amountQuad = (n - restSingle) / 4; 
	
	// how many full float4 values among the global work-items, assumes amountQuad >> totalWorkItems
	int restQuad = amountQuad % totalWorkItems;
	if (restQuad == 0)
		work_item_domain = amountQuad / totalWorkItems;
	else{
		work_item_domain = (amountQuad - restQuad) / totalWorkItems;
	}
	
	// find where to start at each work-item
	int ctaStart = get_global_id(0) * work_item_domain;
	// define the end
	int ctaEnd = ctaStart + work_item_domain;
	
	// Read the data, insert a #PRAGMA unroll if available
	for (int i = ctaStart; i < ctaEnd; i++)
	{  
		y[i] = y[i] + a * x[i];
	}
	
	int lastQuad = ctaEnd + restQuad;
	
	// handle last overshooting float4 values (max totalWorkItems - 1)
	// this part should be improved to a more even distribution among all work-items(?)
	if (restQuad != 0 && (get_global_id(0) + 1 == totalWorkItems)){
		for (int i = ctaEnd; i < lastQuad; i++)
		{	
			y[i] = y[i] + a * x[i];
		}
	}
	
	// handle the last overshooting float values (max 3)
	if (restSingle != 0 && (get_global_id(0) + 1 == totalWorkItems)){
		if (restSingle == 1)
			y[lastQuad].x = y[lastQuad].x + a * x[lastQuad].x;
		else if (restSingle == 2){
			y[lastQuad].x = y[lastQuad].x + a * x[lastQuad].x;
			y[lastQuad].y = y[lastQuad].y + a * x[lastQuad].y;
		}
		else if (restSingle == 3){
			y[lastQuad].x = y[lastQuad].x + a * x[lastQuad].x;
			y[lastQuad].y = y[lastQuad].y + a * x[lastQuad].y;
			y[lastQuad].z = y[lastQuad].z + a * x[lastQuad].z;
		}
	}
	
}