/*
 *  kernel_sspmv_csr4_a1_b0.cl
 *  
 *
 * csr port
 *
 */
 
#pragma OPENCL EXTENSION all : enable

__kernel void kernel_sspmv_csr4_a1_b0(
		const int rows,
		__global const int* d_ptr,
		__global const int4* d_idx,
		__global const float4* d_val,
		__global const float* d_x,
		__global float* d_y
)
{
	/* Starting point for this block */
	int ctaStart = get_group_id(0) * get_local_size(0);
	 
	/* Total no. of threads in the kernel */
	int totalThreads = get_global_size(0);
	
	/* Get current thread */
	int tx = get_local_id(0);

	/* Read the data*/	
	float sum;
	float4 val, x;
	int4 idx;
	
	int iRowBeg, iRowEnd;
	
	for (int i = ctaStart + tx; i < rows; i += totalThreads)
	{ 	
		// Read the beginning and end of the row 
		// which will be processed by this thread
		iRowBeg = d_ptr[i] - 1;
		iRowEnd = d_ptr[i+1] - 1;
		
		/// Read and sum for the column vectors
		sum = 0;
		for (int j = iRowBeg / 4; j < iRowEnd / 4; j++) {
			
			idx = d_idx[j];
			idx -= 1;
			
			x = (float4)(d_x[idx.s0], d_x[idx.s1], d_x[idx.s2], d_x[idx.s3]);
			val = d_val[j];

			sum += dot(x, val);
		}
		
		// Write the result to global memory
		d_y[i] = sum;
	}
}