/*
 *  kernel_sspmv_csr.cl
 *  
 *
 * csr port
 *
 */
 
#define USE_KAHAN_IN_SPMV 0

__kernel void kernel_sspmv_csr_a1(
		const int rows,
		__global const int* d_ptr,
		__global const int* d_idx,
		__global const float* d_val,
		__global const float* d_x,
		__global	 float* d_y
)
{
	/* Starting point for this block */
	int ctaStart = get_group_id(0) * get_local_size(0);
	 
	/* Total no. of threads in the kernel */
	int totalThreads = get_global_size(0);
	
	/* Get current thread */
	int tx = get_local_id(0);

	/* Read the data*/
	for (int i = ctaStart + tx; i < rows; i += totalThreads)
	{ 	
		/* Read the beginning and end of the row 
		 * which will be processed by this thread */
		int iRowBeg = d_ptr[i] - 1;
		int iRowEnd = d_ptr[i+1] - 1;
		
		/* Read and sum for the column vectors */
		#if USE_KAHAN_IN_SPMV
			float sum = d_val[iRowBeg] * d_x[(d_idx[iRowBeg]) - 1];
			float c = 0.0;
			for (int j = iRowBeg + 1; j < iRowEnd; j++) {
				float y = d_val[j] * d_x[(d_idx[j]) - 1] - c;
				float t = sum + y;
	        	c = (t - sum) - y;
	        	sum = t;
			}
		#else		
			float sum = 0;		
			for (int j = iRowBeg; j < iRowEnd; j++) {
				sum += d_val[j] * d_x[(d_idx[j]) - 1];
			}
		#endif

		/* Write the result to global memory */
		d_y[i] += sum;		 	
	}
}