/**
 * @file blas/mp/blas1_mp.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the extension of the
 * BLAS1 routines to MP vectors
 * 	 
 */ 

#include "../blas.h"
#include "../../mat_vec/mp/vector_mp.h"

#ifndef CUKR_BLAS_MP_H_
#define CUKR_BLAS_MP_H_

// ------------------------------------------------------------------- //

/**
 * \brief Dot product between two MP
 *        vectors
 * \param blas The blas used for the operation
 * \param x,y The MP vectors
 */
CUKR_DOUBLE CukrDOT_MP(CUKR_BLAS blas, CUKR_VECTOR_MP x, CUKR_VECTOR_MP y);

// ------------------------------------------------------------------- //

/**
 * \brief AXPY (ax + y) between two MP
 *        vectors
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The MP vectors
 */
void CukrAXPY_MP(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR_MP x, CUKR_VECTOR_MP y);

// ------------------------------------------------------------------- //

/**
 * \brief AYPX (ay + x) bewteen two MP
 *        vectors
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The MP vectors
 */
void CukrAYPX_MP(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR_MP x, CUKR_VECTOR_MP y);

// ------------------------------------------------------------------- //

/**
 * \brief MP vector scaling 
 * \param blas The blas used for the operation
 * \param a The scaling constant
 * \param x The MP vector being scaled
 */
void CukrSCAL_MP(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR_MP x);

// ------------------------------------------------------------------- //

/**
 * \brief MP vector copy
 * \param blas The blas used for the operation
 * \param x The source MP vector
 * \param y The destination MP vector
 */
void CukrCOPY_MP(CUKR_BLAS blas, CUKR_VECTOR_MP x, CUKR_VECTOR_MP y);

// ------------------------------------------------------------------- //


#endif /*CUKR_BLAS_MP_H_*/
