/**
 * @file blas/mp/blas2_mp.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to BLAS2 routines.
 * MP version.
 * 
 * The difference between these routines 
 * and the ones in blas.h is that these
 * are written as function of abstract
 * vectors and matrices, therefore work
 * independent of format, type and
 * location.
 * 	 
 */

#ifndef CUKR_BLAS2_MP_H_
#define CUKR_BLAS2_MP_H_

#include "../blas.h"
#include "../../mat_vec/mp/vector_mp.h"
#include "../../mat_vec/mp/matrix_mp.h"
// ------------------------------------------------------------------- //

/**
 * \brief Sparse matrix vector multiplication
 * \param blas The blas user for the operation
 * \param alpha Scaling factor for Ax
 * \param mat The MP matrix
 * \param x MP vector being multiplied
 * \param beta Scaling factor for y
 * \param y Result MP vector
 */ 
void CukrSPMV_MP(CUKR_BLAS blas, CUKR_DOUBLE alpha, CUKR_MATRIX_MP mat, 
                 CUKR_VECTOR_MP x, CUKR_DOUBLE beta, CUKR_VECTOR_MP y);



#endif /*CUKR_BLAS2_MP_H_*/
