/**
 * @file blas/blas1_mg.c
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to BLAS1 routines.
 * 
 * The difference between these routines 
 * and the ones in blas.h is that these
 * are written as function of abstract
 * vectors and matrices, therefor work
 * independent of format, type and
 * location.
 * 	 
 */

#ifndef CUKR_BLAS1_SP_H_
#define CUKR_BLAS1_SP_H_

#include "../../mat_vec/sp/vector_sp.h"
#include "../blas.h"

/* 
 * Function pointers for the functions which
 * have to be scheduled to threads
 */

typedef CUKR_DOUBLE (*CUKR_BLAS1_DOT_SP) (CUKR_BLAS, CUKR_VECTOR_SP, CUKR_VECTOR_SP);
typedef void        (*CUKR_BLAS1_AXPY_SP)(CUKR_BLAS, CUKR_DOUBLE, CUKR_VECTOR_SP, CUKR_VECTOR_SP);
typedef void        (*CUKR_BLAS1_AYPX_SP)(CUKR_BLAS, CUKR_DOUBLE, CUKR_VECTOR_SP, CUKR_VECTOR_SP);
typedef void        (*CUKR_BLAS1_SCAL_SP)(CUKR_BLAS, CUKR_DOUBLE, CUKR_VECTOR_SP);
typedef void        (*CUKR_BLAS1_COPY_SP)(CUKR_BLAS, CUKR_VECTOR_SP, CUKR_VECTOR_SP);

// ------------------------------------------------------------------- //

/**
 * \brief Dot product
 * \param blas The blas used for the operation
 * \param x,y The vectors
 */
CUKR_DOUBLE CukrDOT_SP(CUKR_BLAS blas, CUKR_VECTOR_SP x, CUKR_VECTOR_SP y);

// ------------------------------------------------------------------- //

/**
 * \brief AXPY (ax + y)
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The vectors
 */
void CukrAXPY_SP(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR_SP x, CUKR_VECTOR_SP y);

// ------------------------------------------------------------------- //

/**
 * \brief AYPX (ay + x)
 * \param blas The blas used for the operation
 * \param a The constant
 * \param x,y The vectors
 */
void CukrAYPX_SP(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR_SP x, CUKR_VECTOR_SP y);

// ------------------------------------------------------------------- //

/**
 * \brief Vector scaling 
 * \param blas The blas used for the operation
 * \param a The scaling constant
 * \param x The vectors being scaled
 */
void CukrSCAL_SP(CUKR_BLAS blas, CUKR_DOUBLE a, CUKR_VECTOR_SP x);

// ------------------------------------------------------------------- //

/**
 * \brief Vector copy
 * \param blas The blas used for the operation
 * \param x The source vector
 * \param y The destination vector
 */
void CukrCOPY_SP(CUKR_BLAS blas, CUKR_VECTOR_SP x, CUKR_VECTOR_SP y);

// ------------------------------------------------------------------- //


#endif /*CUKR_BLAS_SP_H_*/
