/**
 * @file blas/sp/blas2.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to BLAS2 routines.
 * 
 * The difference between these routines 
 * and the ones in blas.h is that these
 * are written as function of abstract
 * vectors and matrices, therefor work
 * independent of format, type and
 * location.
 * 	 
 */

#ifndef CUKR_BLAS2_SP_H_
#define CUKR_BLAS2_SP_H_

#include "../blas.h"
#include "../../mat_vec/sp/vector_sp.h"
#include "../../mat_vec/sp/matrix_sp.h"

/* 
 * Function pointers for the functions which
 * have to be scheduled to threads
 */
 
typedef void (*CUKR_BLAS2_SPMV_SP)(CUKR_BLAS, CUKR_DOUBLE, CUKR_MATRIX_SP, CUKR_VECTOR_SP, CUKR_DOUBLE, CUKR_VECTOR_SP); 


// ------------------------------------------------------------------- //

/**
 * \brief Sparse matrix vector multiplication
 * \param blas The blas used for the operation
 * \param alpha Scaling factor for Ax
 * \param mat The matrix
 * \param x Vector being multiplied
 * \param beta Scaling factor for y
 * \param y Result vector
 */ 
void CukrSPMV_SP(CUKR_BLAS blas, CUKR_DOUBLE alpha, CUKR_MATRIX_SP mat, 
              CUKR_VECTOR_SP x, CUKR_DOUBLE beta, CUKR_VECTOR_SP y);



#endif /*CUKR_BLAS2_SP_H_*/
