/**
 * @file blas/blasx1.c
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to 
 * SP/MP/MPI BLASX1 routines.
 * 	 
 */

#include "blasx1.h"

/**
/**
 * \brief Pointwise-product between
 *        two vectors
 * \param blas The BLAS structure
 * \param x,y The intput vectors
 * \param z The result vector 
 */
void CukrPWPR(CUKR_BLAS blas, CUKR_VECTOR x, CUKR_VECTOR y, CUKR_VECTOR z)
{
	#ifdef CUKR_MPI
		return CukrPWPR_MPI(blas, x, y, z);
	#else
		#ifdef CUKR_MP
			return CukrPWPR_MP(blas, x, y, z);
		#else
			return CukrPWPR_SP(blas, x, y, z);
		#endif
	#endif		
}

