/**
 * @file blas/blasx1.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to 
 * SP/MP/MPI BLASX1 routines.
 * 	 
 */

#ifndef CUKR_BLASX1_H_
#define CUKR_BLASX1_H_

#include "../mat_vec/vector.h"
#include "../blas/blas.h"
#include "blasx.h"

/* Decide which include file to use,
 * depending on defines set at built time.*/ 
#ifdef CUKR_MPI
	#include "mpi/blasx1_mpi.h"
#else
	#ifdef CUKR_MP
		#include "mp/blasx1_mp.h"
	#else
		#include "sp/blasx1_sp.h"
	#endif
#endif


/**
 * \brief Pointwise-product between
 *        two vectors
 * \param blas The BLAS structure
 * \param x,y The intput vectors
 * \param z The result vector 
 */
void CukrPWPR(CUKR_BLAS blas, CUKR_VECTOR x, CUKR_VECTOR y, CUKR_VECTOR z);


#endif /*CUKR_BLASX_H_*/
