/**
 * @file blasx/generic/generic_blasx.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * This defines the "generic blasx", with
 * all functions written here. It does not
 * use any libraries.
 */

#ifndef CUKR_GENERIC_BLASX_H_
#define CUKR_GENERIC_BLASX_H_
 
 
#include "../../blasx.h"

// ------------------------------------------------------------------- //

/**
 * \brief Wrapper for DPWPR (pointwise
 *        product in double precision) 
 * \param blas The BLASX structure
 * \param n The vector size
 * \param x,y The intput vectors
 * \param z The result vector 
 */
void generic_dpwpr(CUKR_INT n, CUKR_DOUBLE* x, CUKR_DOUBLE* y, CUKR_DOUBLE* z); 

/**
 * \brief Wrapper for QPWPR (pointwise
 *        product in quasi-double precision) 
 * \param blas The BLASX structure
 * \param n The vector size
 * \param xh, xt, yh, yt The intput vectors 
 *        (heads & tails)
 * \param zh, zt The result vector (head & tail)
 */
void generic_qpwpr(CUKR_INT n, CUKR_SINGLE* xh, CUKR_SINGLE* xt,
				 			   CUKR_SINGLE* yh, CUKR_SINGLE* yt,
							   CUKR_SINGLE* zh, CUKR_SINGLE* zt);
							   
							    
/**
 * \brief Wrapper for SPWPR (pointwise
 *        product in single precision) 
 * \param blas The BLASX structure
 * \param n The vector size
 * \param x,y The intput vectors
 * \param z The result vector 
 */
void generic_spwpr(CUKR_INT n, CUKR_SINGLE* x, CUKR_SINGLE* y, CUKR_SINGLE* z); 
    


// ------------------------------------------------------------------- //

/**
 * \brief Create a generic blas object by
 *        binding the above functions to a
 * 		  CUKR_BLASX structure 
 * \param blasx pointer to a blasx structure
 */
void blasx_create_generic(CUKR_BLASX blasx);                


#endif /*CUKR_GENERIC_BLASX_H_*/               
