/**
 * @file blas/gpu/cuda/kernels.cu
 * @author Serban Georgescu
 * @since 2008
 * 
 * Acts as a bridge between the C++ CUDA
 * implementation and the rest of the C 
 * files. All the CUDA BLASX kernels must be 
 * included here.
 * 
 * The following kernels are included:
 *  - spwpr
 * 	- qpwpr
 */

#ifndef CUKR_BLASX_GPUBLAS_KERNELS_H_
#define CUKR_BLASX_GPUBLAS_KERNELS_H_


// ****************************  FOR SINGLE  ***************************** //
 

/**
 * \brief SPRPW kernel in single, on the GPU
 * \param n Vector size
 * \param a The constant
 * \param d_x Device pointer to the first vector
 * \param d_y Device pointer to the second vector
 * \param d_z Device pointer to the result vector
 */ 
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_spwpr(int n, float *d_x, float *d_y, float *d_z);


// ****************************  FOR DOUBLE  ***************************** //
 

/**
 * \brief DPRPW kernel in double, on the GPU
 * \param n Vector size
 * \param a The constant
 * \param d_x Device pointer to the first vector
 * \param d_y Device pointer to the second vector
 * \param d_z Device pointer to the result vector
 */ 
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_dpwpr(int n, double *d_x, double *d_y, double *d_z);
					   	      
// ****************************  FOR QDOUBLE  ***************************** //
 

/**
 * \brief QPRPW kernel in double-single,
 *        on the GPU
 * \param n Vector size
 * \param a The constant
 * \param d_xh, d_xt Device pointers to the 
 *        first vector (head and tail).
 * \param d_yh, d_yt Device pointers to the 
 *        second vector (head and tail).
 * \param d_zh, d_zt Device pointers to the 
 *        result vector (head and tail).
 */ 
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_qpwpr(int n, float *d_xh, float *d_xt, 
					   	      float *d_yh, float *d_yt,
					   	      float *d_zh, float *d_zt);					   	      
   

#endif /*CUKR_GPUBLAS_KERNELS_H_*/
