/**
 * @file blasx/gpu/cuda/kernels_config.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Contains configuration stuff for the
 * CUDA kernels (e.g. threads, blocks, etc).
 * 
 */

#ifndef BLASX_KERNELS_CONFIG_H_
#define BLASX_KERNELS_CONFIG_H_


// ****************************  FOR SINGLE  ***************************** //

/**
 * Number of thread blocks to be used
 * for the single precision PWPR kernel 
 */
#define CUKR_SPWPR_CTAS  256
/** 
 * Number of threads per block to be
 * used for the single precision PWPR kernel 
 */
#define CUKR_SPWPR_THREADS 192


// ****************************  FOR DOUBLE  ***************************** //

/**
 * Number of thread blocks to be used
 * for the double precision PWPR kernel 
 */
#define CUKR_DPWPR_CTAS  256
/** 
 * Number of threads per block to be
 * used for the double PWPR kernel 
 */
#define CUKR_DPWPR_THREADS 192




// *************************  FOR SINGLE-DOUBLE  ************************** //


/**
 * Number of thread blocks to be used
 * for the double-single PWPR kernel 
 */
#define CUKR_QPWPR_CTAS  256
/** 
 * Number of threads per block to be
 * used for the double-single PWPR kernel 
 */
#define CUKR_QPWPR_THREADS 192


// -------------------------------------------------------------- //

#endif /*BLASX_KERNELS_CONFIG_H_*/
