/**
 * @file blasx/gpu/cuda/kernels_double.cu
 * @author Serban Georgescu
 * @since 2008
 * 
 * Acts as a bridge between the C++ CUDA
 * implementation and the rest of the C 
 * files. All the CUDA single precision
 * kernels must be included here.
 * 
 * The following kernels are included:
 *  - dpwpr
 */

#include <stdio.h>
#include <cuda.h>
#include <cutil.h>
#include <cublas.h>

#include "kernels_config.h"
#include "double/kernel_dpwpr.h"


/**
 * \brief DPRPW kernel in single, on the GPU
 * \param n Vector size
 * \param a The constant
 * \param d_x Device pointer to the first vector
 * \param d_y Device pointer to the second vector
 * \param d_z Device pointer to the result vector
 */ 
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_dpwpr(int n, double *d_x, double *d_y, double *d_z)
{	
	/* Preparations */
	int dynMem = 2 * CUKR_SPWPR_THREADS * sizeof(double); 
	
	/* Kernel invocation */
	kernel_dpwpr<<<CUKR_DPWPR_CTAS, CUKR_DPWPR_THREADS, dynMem>>> (d_x, d_y, d_z, n);  
	CUT_CHECK_ERROR("kernel_dpwpr() execution failed\n");
	
	/* Do a syncronize, for timing */
    CUDA_SAFE_CALL(cudaThreadSynchronize());   		
}


