/**
 * @file blas/gpu/cuda/kernels_qdouble.cu
 * @author Serban Georgescu
 * @since 2008
 * 
 * Acts as a bridge between the C++ CUDA
 * implementation and the rest of the C 
 * files. All the CUDA single-double
 * (qdouble, emulated-double) precision
 * kernels must be included here.
 * 
 * The following kernels are included:
 *  - qpwpr
 */

#include <stdio.h>
#include <cuda.h>
#include <cutil.h>
#include <cublas.h>

#include "../../../../cukr.h" 
#include "kernels_config.h"
#include "qdouble/kernel_qpwpr.h"


//========================== FORWARD DECLARATIONS ========================= //

void HandleCudaMallocErrors(cublasStatus status);



//============================ PUBLIC FUNCTIONS =========================== //


/**
 * \brief QPRPW kernel in double-single,
 *        on the GPU
 * \param n Vector size
 * \param a The constant
 * \param d_xh, d_xt Device pointers to the 
 *        first vector (head and tail).
 * \param d_yh, d_yt Device pointers to the 
 *        second vector (head and tail).
 * \param d_zh, d_zt Device pointers to the 
 *        result vector (head and tail).
 */ 
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_qpwpr(int n, float *d_xh, float *d_xt, 
					   	      float *d_yh, float *d_yt,
					   	      float *d_zh, float *d_zt)
{
	/* Preparations */
	int dynMem = 4 * CUKR_QPWPR_THREADS * sizeof(float); 

	/* Kernel invocation */
	kernel_qpwpr<<<CUKR_QPWPR_CTAS, CUKR_QPWPR_THREADS, dynMem>>> (d_xh, d_xt, d_yh, d_yt, d_zh, d_zt, n);
	CUT_CHECK_ERROR("kernel_qpwpr() execution failed\n"); 
	
	/* Do a syncronize, for timing */
    CUDA_SAFE_CALL(cudaThreadSynchronize());  	               
	
}					   	     	   	      
