/**
 * @file blasx/gpu/cuda/kernels_single.cu
 * @author Serban Georgescu
 * @since 2008
 * 
 * Acts as a bridge between the C++ CUDA
 * implementation and the rest of the C 
 * files. All the CUDA single precision
 * kernels must be included here.
 * 
 * The following kernels are included:
 *  - spwpr
 */

#include <stdio.h>
#include <cuda.h>
#include <cutil.h>
#include <cublas.h>

//#include "../../../../include/cukr.h" 
#include "kernels_config.h"
#include "single/kernel_spwpr.h"


/**
 * \brief SPRPW kernel in single, on the GPU
 * \param n Vector size
 * \param a The constant
 * \param d_x Device pointer to the first vector
 * \param d_y Device pointer to the second vector
 * \param d_z Device pointer to the result vector
 */ 
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_spwpr(int n, float *d_x, float *d_y, float *d_z)
{	
	/* Preparations */
	int dynMem = 2 * CUKR_SPWPR_THREADS * sizeof(float); 
	
	/* Kernel invocation */
	kernel_spwpr<<<CUKR_SPWPR_CTAS, CUKR_SPWPR_THREADS, dynMem>>> (d_x, d_y, d_z, n);  
	CUT_CHECK_ERROR("kernel_spwpr() execution failed\n");
	
	/* Do a syncronize, for timing */
    CUDA_SAFE_CALL(cudaThreadSynchronize());   		
}


