/**
 * @file blas/gpu/gpu_blasx.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Defines wrappers for BLAS functions 
 * on the GPU. 
 * 
 * The functions take as parameter device
 * pointers, so they only act on data that
 * has been PREVIOUSLY COPIED to the device.
 * 
 */

#ifndef GPU_BLASX_H_
#define GPU_BLASX_H_

#include "../../blasx.h"

/**
 * \brief Wrapper for QPWPR (pointwise
 *        product in quasi-double precision) 
 * \param blas The BLASX structure
 * \param n The vector size
 * \param xh, xt, yh, yt The intput vectors 
 *        (heads & tails)
 * \param zh, zt The result vector (head & tail)
 */
void gpu_qpwpr(CUKR_INT n, CUKR_SINGLE* xh, CUKR_SINGLE* xt,
				 			   CUKR_SINGLE* yh, CUKR_SINGLE* yt,
							   CUKR_SINGLE* zh, CUKR_SINGLE* zt);
							   
							    
/**
 * \brief Wrapper for SPWPR (pointwise
 *        product in single precision) 
 * \param blas The BLASX structure
 * \param n The vector size
 * \param x,y The intput vectors
 * \param z The result vector 
 */
void gpu_spwpr(CUKR_INT n, CUKR_SINGLE* x, CUKR_SINGLE* y, CUKR_SINGLE* z); 

// ------------------------------------------------------------------- //

/*
 * \brief Create a generic blasx object by
 *        binding the above functions to a
 * 		  CUKR_BLASX structure 
 * \param blas pointer to a blas structure
 */
void blasx_create_gpu(CUKR_BLASX blasx);

#endif /*GENERIC_BLASX_H_*/               
