/**
 * @file blasx/blasx1.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the interface to BLASX1 routines.
 * 
 * The difference between these routines 
 * and the ones in blasx.h is that these
 * are written as function of abstract
 * vectors and matrices, therefor work
 * independent of format, type and
 * location.
 * 	
 */

#ifndef CUKR_BLASX1_SP_H_
#define CUKR_BLASX1_SP_H_

#include "../../mat_vec/sp/vector_sp.h"
#include "../../blas/blas.h"
#include "../blasx.h"

/* 
 * Function pointers for the functions which
 * have to be scheduled to threads
 */

typedef void (*CUKR_BLASX1_PWPR_SP)(CUKR_BLAS, CUKR_VECTOR_SP, CUKR_VECTOR_SP, CUKR_VECTOR_SP);

// ------------------------------------------------------------------- //

/**
 * \brief Pointwise-product between
 *        two vectors
 * \param blas The BLAS structure
 * \param x,y The intput vectors
 * \param z The result vector 
 */
void CukrPWPR_SP(CUKR_BLAS blas, CUKR_VECTOR_SP x, CUKR_VECTOR_SP y, CUKR_VECTOR_SP z);


#endif /*CUKR_BLASX_SP_H_*/
