/**
 * @file final/final.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Here we include all the post tasks
 * that must be performed, including:
 * 	- reporting the BLAS performance
 *  - ...
 */

#ifndef FINAL_H_
#define FINAL_H_

#include "../solvers/solver.h" 


/**
 * \brief Final cleanup, shutdown and reporting
 * \param opts Options  
 */
void CukrFinalize(int opts);

/**
 * \brief Generate performance reports for
 *        the BLAS 
 */
void CukrFin_ReportBLAS();

/**
 * \brief Generate solver report
 * \param solver The solver
 */
void CukrFin_ReportSolver(CUKR_SOLVER solver);

/**
 * \brief Generate runtime report
 */
void CukrFin_ReportRuntime();

/**
 * \brief Shutdown CUBLAS
 */
void CukrFin_ShutdownCublas();

/**
 * \brief Shutdown CUBLAS
 */
void CukrFin_ShutdownOpenCL();

/**
 * \brief Stop all worker threads
 */
void CukrFin_ShutdownThreading();

/**
 * \brief Shutdown MPI 
 */
void CukrFin_ShutdownMPI();

#endif /*FINAL_H_*/
