/*
 *  init_opencl.c
 *  
 *
 *  Created by Olav Aanes Fagerlund, winter 2010.
 *
 */

#include "init_opencl.h"

static char *
load_source(const char *filename)
{
    struct stat statbuf;
    FILE        *fh;
    char        *source;
	
    fh = fopen(filename, "r");
    if (fh == 0)
        return 0;
	
    stat(filename, &statbuf);
    source = (char *) malloc(statbuf.st_size + 1);
    fread(source, statbuf.st_size, 1, fh);
    source[statbuf.st_size] = '\0';
	
    return source;
}

int loadAndBuild(const char *file, char *kernel){
	/* Load kernel sources into memory */
	char *kernel_src = load_source(file);

	/* Create the compute program memory object */
	ComputeProgram = clCreateProgramWithSource(ComputeContext, 1, (const char **) &kernel_src, NULL, &err);
	if (!ComputeProgram || err != CL_SUCCESS)
    {
        printf("Error: Failed to create compute program for file %s!\n", file);
        return EXIT_FAILURE;
    }
	else {
		printf("CL compute program memory object set up successfully!\n");
	}
	
	/* Build the executable and add to the compute program memory object */
	err = clBuildProgram(ComputeProgram, 0, NULL, /*"-cl-finite-math-only"*/NULL, NULL, NULL);
	if (err == CL_BUILD_PROGRAM_FAILURE) {
		printf("JA!\n");
	}
	else if (err != CL_SUCCESS)
	{
		printf("Error: Failed build program for kernel %s!\n %d\n", kernel, err);
	}
	else {
		printf("CL compute program source for %s built successfully!\n", kernel);
	}

	if (kernel == "kernels_qdouble"){
		
		ComputeKernel_qdot = clCreateKernel(ComputeProgram, "kernel_qdot", &err);
		if (!ComputeKernel_qdot || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qdot");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
		
		ComputeKernel_qscal = clCreateKernel(ComputeProgram, "kernel_qscal", &err);
		if (!ComputeKernel_qscal || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qscal");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
		
		ComputeKernel_qaxpy = clCreateKernel(ComputeProgram, "kernel_qaxpy", &err);
		if (!ComputeKernel_qaxpy || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qaxpy");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
		
		ComputeKernel_qaypx = clCreateKernel(ComputeProgram, "kernel_qaypx", &err);
		if (!ComputeKernel_qaypx || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qaypx");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
		
		ComputeKernel_qspmv_csr = clCreateKernel(ComputeProgram, "kernel_qspmv_csr", &err);
		if (!ComputeKernel_qspmv_csr || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}

		ComputeKernel_qspmv_csr_a1 = clCreateKernel(ComputeProgram, "kernel_qspmv_csr_a1", &err);
		if (!ComputeKernel_qspmv_csr_a1 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr_a1");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}

		ComputeKernel_qspmv_csr_a1_b0 = clCreateKernel(ComputeProgram, "kernel_qspmv_csr_a1_b0", &err);
		if (!ComputeKernel_qspmv_csr_a1_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr_a1_b0");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}

		ComputeKernel_qspmv_csr_b0 = clCreateKernel(ComputeProgram, "kernel_qspmv_csr_b0", &err);
		if (!ComputeKernel_qspmv_csr_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr_b0");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}

		ComputeKernel_qspmv_csr4 = clCreateKernel(ComputeProgram, "kernel_qspmv_csr4", &err);
		if (!ComputeKernel_qspmv_csr4 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr4");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}

		ComputeKernel_qspmv_csr4_a1 = clCreateKernel(ComputeProgram, "kernel_qspmv_csr4_a1", &err);
		if (!ComputeKernel_qspmv_csr4_a1 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr4_a1");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	
		ComputeKernel_qspmv_csr4_a1_b0 = clCreateKernel(ComputeProgram, "kernel_qspmv_csr4_a1_b0", &err);
		if (!ComputeKernel_qspmv_csr4_a1_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr4_a1_b0");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	
		ComputeKernel_qspmv_csr4_b0 = clCreateKernel(ComputeProgram, "kernel_qspmv_csr4_b0", &err);
		if (!ComputeKernel_qspmv_csr4_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_csr4_b0");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
		
		ComputeKernel_qspmv_ell = clCreateKernel(ComputeProgram, "kernel_qspmv_ell", &err);
		if (!ComputeKernel_qspmv_ell || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", "kernel_qspmv_ell");
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sdot"){
		ComputeKernel_sdot = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sdot || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
		
		size_t auto_local;
		
		// Set workgroup sizes
		err = clGetKernelWorkGroupInfo(ComputeKernel_sdot, ComputeDeviceId, CL_KERNEL_WORK_GROUP_SIZE, sizeof(size_t), &auto_local, NULL);
		if (err != CL_SUCCESS) {
			printf("clGetKernelWorkGroupInfo failed! : %d\n", err);
		}
		AUTO_LOCAL_SIZE_SDOT = auto_local;
	}
	
	else if (kernel == "kernel_ddot"){
		ComputeKernel_ddot = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_ddot || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sscal"){
		ComputeKernel_sscal = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sscal || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_dscal"){
		ComputeKernel_dscal = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_dscal || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_saxpy"){
		ComputeKernel_saxpy = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_saxpy || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_daxpy"){
		ComputeKernel_daxpy = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_daxpy || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_saypx"){
		ComputeKernel_saypx = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_saypx || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_daypx"){
		ComputeKernel_daypx = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_daypx || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_csr"){
		ComputeKernel_sspmv_csr = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_csr_a1"){
		ComputeKernel_sspmv_csr_a1 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr_a1 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_csr_a1_b0"){
		ComputeKernel_sspmv_csr_a1_b0 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr_a1_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_csr_b0"){
		ComputeKernel_sspmv_csr_b0 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	else if (kernel == "kernel_sspmv_csr4"){
		ComputeKernel_sspmv_csr4 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr4 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_csr4_a1"){
		ComputeKernel_sspmv_csr4_a1 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr4_a1 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_csr4_a1_b0"){
		ComputeKernel_sspmv_csr4_a1_b0 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr4_a1_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_csr4_b0"){
		ComputeKernel_sspmv_csr4_b0 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_csr4_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_sspmv_ell"){
		ComputeKernel_sspmv_ell = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_sspmv_ell || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	// SPMV double ones
	else if (kernel == "kernel_dspmv_ell"){
		ComputeKernel_dspmv_ell = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_dspmv_ell || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_dspmv_csr"){
		ComputeKernel_dspmv_csr = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_dspmv_csr || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_dspmv_csr_b0"){
		ComputeKernel_dspmv_csr_b0 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_dspmv_csr_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_dspmv_csr4"){
		ComputeKernel_dspmv_csr4 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_dspmv_csr4 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
	
	else if (kernel == "kernel_dspmv_csr4_b0"){
		ComputeKernel_dspmv_csr4_b0 = clCreateKernel(ComputeProgram, kernel, &err);
		if (!ComputeKernel_dspmv_csr4_b0 || err != CL_SUCCESS)
		{
			printf("Error: Failed to create compute kernel %s!\n", kernel);
		}
		else {
			printf("CL compute kernel memory object set up successfully!\n");
		}
	}
}

int CukrInit_OpenCL_cl(){
	unsigned int num_devices_to_use = 1;
	unsigned int size_comp_dev_id = 1;
	
	unsigned int num_platforms = 0;
	unsigned int num_devices = 0;
	
	/* Find the OpenCL platform */
	err = clGetPlatformIDs(1, &ComputePlatformId, &num_platforms);
	if (err != CL_SUCCESS)
    {
        printf("Error: Failed to get platform IDs! %d\n", err);
        return EXIT_FAILURE;
    }
	
	/* Find a GPU device */
	err = clGetDeviceIDs(ComputePlatformId, CL_DEVICE_TYPE_GPU, size_comp_dev_id, &ComputeDeviceId, &num_devices);
	if (err != CL_SUCCESS)
    {
        printf("Error: Failed to get device IDs! %d\n", err);
        return EXIT_FAILURE;
    }
	printf("\n* * * %d OpenCL devices found in the system * * *\n", num_devices);
	
	/* Create a compute context using the found OpenCL device */
	ComputeContext = clCreateContext(NULL, num_devices_to_use, &ComputeDeviceId, NULL, NULL, &err);
	if (err != CL_SUCCESS)
    {
        printf("Error: Failed to create compute context! %d\n", err);
        return EXIT_FAILURE;
    }
	else {
		printf("CL compute context set up successfully!\n");
	}

	/* Create the command queue */
	ComputeCommands = clCreateCommandQueue(ComputeContext, ComputeDeviceId, /*CL_QUEUE_PROFILING_ENABLE*/0, &err);
	if (err != CL_SUCCESS)
	{
		printf("clCreateCommandQueue failed\n %d\n", err);
		return -1;
	}
	
	// Load and build all qdouble kernels (in same file as they depend on same ds_ops)
	loadAndBuild("src/kernels_qdouble.cl", "kernels_qdouble");
	
	loadAndBuild("src/kernel_saxpy.cl", "kernel_saxpy");
	loadAndBuild("src/kernel_daxpy.cl", "kernel_daxpy");
	
	loadAndBuild("src/kernel_saypx.cl", "kernel_saypx");
	loadAndBuild("src/kernel_daypx.cl", "kernel_daypx");
	
	
	loadAndBuild("src/kernel_sdot.cl", "kernel_sdot");
	loadAndBuild("src/kernel_ddot.cl", "kernel_ddot");
	
	loadAndBuild("src/kernel_sscal.cl", "kernel_sscal");
	loadAndBuild("src/kernel_dscal.cl", "kernel_dscal");
	
	loadAndBuild("src/kernel_sspmv_csr.cl", "kernel_sspmv_csr");
	loadAndBuild("src/kernel_sspmv_csr_a1.cl", "kernel_sspmv_csr_a1");
	loadAndBuild("src/kernel_sspmv_csr_a1_b0.cl", "kernel_sspmv_csr_a1_b0");
	loadAndBuild("src/kernel_sspmv_csr_b0.cl", "kernel_sspmv_csr_b0");
	
	loadAndBuild("src/kernel_sspmv_csr4.cl", "kernel_sspmv_csr4");
	loadAndBuild("src/kernel_sspmv_csr4_a1.cl", "kernel_sspmv_csr4_a1");
	loadAndBuild("src/kernel_sspmv_csr4_a1_b0.cl", "kernel_sspmv_csr4_a1_b0");
	loadAndBuild("src/kernel_sspmv_csr4_b0.cl", "kernel_sspmv_csr4_b0");

	
	loadAndBuild("src/kernel_dspmv_csr.cl", "kernel_dspmv_csr");
	loadAndBuild("src/kernel_dspmv_csr_b0.cl", "kernel_dspmv_csr_b0");

	loadAndBuild("src/kernel_dspmv_csr4.cl", "kernel_dspmv_csr4");
	loadAndBuild("src/kernel_dspmv_csr4_b0.cl", "kernel_dspmv_csr4_b0");
	
	loadAndBuild("src/kernel_sspmv_ell.cl", "kernel_sspmv_ell");
	loadAndBuild("src/kernel_dspmv_ell.cl", "kernel_dspmv_ell");
	
	return num_devices;
}
