/**
 * @file main.c
 * @author Serban Georgescu
 * @since 2008
 * 
 * In the main.c we do the following:
 * 	- init the solver
 *  - parse command line options
 *  - ...
 */ 
    
#define LOGGER_NAME "CUKr"

/* Standard includes */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* CUKr include */
#include "cukr_all.h"

/* Get access to the option database */
extern CukrDB cukr_db;
 

/********************************************************* 
* The actual program 
*********************************************************/

/**
 * \brief The actual work being done has to
 *        be put here.
 */
void work() 
{	
		
	CUKR_VECTOR x = NULL, b = NULL;
	//CUKR_VECTOR_SP x = NULL, b = NULL;
	CUKR_SOLVER s;
	/*
	int pe = cukr_db.pe->ival[0];
	int i, j, n = 600000;
	struct timeval t_start, t_end;
	struct timezone t_zone; 
	CukrVecspCreate(&x, 10000, CUKR_TYPE_DOUBLE, "x");
	CLOCK_START;
	for (i = 0; i < n; i++) {
		for (j = 0; j < pe; j++)
			CukrThreadsRun(j, (void*)CukrDOT_SP, CUKR_FUNCID_DOT, cukr_db.blasD, x, x);
			//CukrThreadsRun(j, NULL, CUKR_FUNCID_DUMMY);
		CukrThreadsWaitCompletion(pe); 
	}
	CLOCK_END;
	double elapsed = CLOCK_SPAN;
	printf("Time = %g\n", elapsed);
	
	return ;
	*/
	
	/* Create x and rhs
	 * The solution vector has to be shared */
	//CukrVecmpiCreate(&x, cukr_db.system_mat->cols, CUKR_TYPE_SINGLE, CUKR_SHTYPE_SHARED, "x");
	//CukrVecmpiCreate(&b, cukr_db.system_mat->rows, CUKR_TYPE_SINGLE, CUKR_SHTYPE_DISTRIBUTED, "b");
	
	/* Set x = 0 and b = 1 */
	CukrVecSetFromVal(x, 0.0, CUKR_DEFAULT);
	CukrVecSetFromVal(b, 1.0, CUKR_DEFAULT);
	
	/* Create solver and solve */
	CukrSolverCreate(&s, "CG");
	//CukrSolverSetOperators(s, cukr_db.system_mat);
	CukrSolverSolve(s, b, x);	
	
	/* Report solver behaviour */
  	CukrFin_ReportSolver(s);
  	
  	/* Destroy solver and vectors */
  	CukrSolverDestroy(&s); 
	CukrVecDestroy(&x);
	CukrVecDestroy(&b);
	
}

/********************************************************* 
* The main function 
*********************************************************/

/**
 * \brief Main function, the root of all evil :)
 * 
 * \param argc argument count
 * \param argv pointer to the parameter list
 */
int main(int argc, char* argv[]) 
{
	/* Initialization */
	CukrInit(argc, argv, CUKR_OPTS_STRICT_SYNTAX);
	CukrStart(CUKR_DEFAULT);
	
	/* Work */
	work();	
	
	/* Finalization */
	CukrFinalize(CUKR_DEFAULT);
	
	return 0;
}
