/**
 * @file mat_vec/matrix_defs.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Holds various matrix constants
 * and defs
 */

#ifndef CUKR_MATRIX_DEFS_H_
#define CUKR_MATRIX_DEFS_H_


/*
 * List of matrix formats and
 * variations. Currently only
 * CSR format is supported, with
 * two variations: CSR and CSR4
 */

/**
 * CSR (Compressed Sparse Row)
 */
#define CUKR_MATRIX_FORMAT_CSR 0
/**
 * No variation
 */
#define CUKR_MATRIX_FORMAT_VARIATION_NONE 10
/**
 * CSR4 (CSR padded so that the number
 * of elements in each row be multiple
 * of 4)
 */
#define CUKR_MATRIX_FORMAT_VARIATION_CSR4 20
/**
 * BCSR (Block CSR)
 * a new format, 2009.
 */
#define CUKR_MATRIX_FORMAT_BCSR 1
/**
 * BCSR4 (BCSR padded so that the number
 * of elements in each block be multiple
 * of 4)
 */
#define CUKR_MATRIX_FORMAT_VARIATION_BCSR4 11
/**
 * HYB (ELLPACK/ITPACK - Coordinate Hybrid)
 */
#define CUKR_MATRIX_FORMAT_HYB 2
/**
 * Block HYB (BHYB)
 */
#define CUKR_MATRIX_FORMAT_BHYB 3
/**
 * BHYB4 (BHYB padded so that the number
 * of elements in each block be multiple
 * of 4)
 */
#define CUKR_MATRIX_FORMAT_VARIATION_BHYB4 13
#endif /*CUKR_MATRIX_DEFS_H_*/
