/**
 * @file mat_vec/matrix_gen.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Interface to the SP/MP/MPI matrix
 * generating functions
 */

#ifndef CUKR_MATRIX_GEN_H_
#define CUKR_MATRIX_GEN_H_

#include "../cukr.h"
#include "matrix.h"


/**
 * \brief Generates a diagonally-dominated sparse
 *        random matrix. The matrix is made from
 *        <band_group_size> groups of bands, separated
 *        by <space_group_size> empty bands. There are
 *        a total of group_count groups.  
 * \param mat Pointer to the matrix 
 * \param n Matrix size
 * \param group_count Number of band groups (symmetric)
 * \param band_group_size How many bands in a group
 * \param space_group_size How many spaces are used
 *        as separators.
 * \param format Matrix format (CSR) 
 * \param type Matrix type (float or default) 
 * \param name Matrix name
 */
void CukrMatGenBanded(CUKR_MATRIX* mat, int n, int group_count, int band_group_size, 
                      int space_group_size, int format, int type, char* name);
                      
                         
#endif /*CUKR_MATRIX_GEN_H_*/
