/**
 * @file mat_vec/sp/bhyb/bhyb_matrix_sp.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains all the functions needed
 * to support the block hybrid matrix format
 */

#ifndef CUKR_BHYB_MATRIX_SP_H_
#define CUKR_BHYB_MATRIX_SP_H_

/* =============== FORWARD DECLARATIONS ============== */

typedef struct _CUKR_BHYB_MATRIX_SP *CUKR_BHYB_MATRIX_SP;

/* =================================================== */

#include "../matrix_sp.h"
#include "../vector_sp.h"

/**
 * Holds data for a matrix of type block Hybrid
 */
struct _CUKR_BHYB_MATRIX_SP{   

	/** The block size */
	CUKR_INT r,c;

	/**
	 * For the BELL part 
	 */
	
	/* Matrix dimensions */
	CUKR_INT bell_nz; 	
	
	/* Number of nonzeros per row */
	CUKR_INT bell_nz_per_row;
	
	/* Read stride */
	CUKR_INT bell_stride;
	
	/* Column vector (bell_nz_per_row * bell_rows length) */
	CUKR_VECTOR_SP bell_idx;
	
	/* Value vector (ell_nz_per_row * bell_rows length) */
	CUKR_VECTOR_SP bell_val;
	
	/*
	 * For the BCSR part
	 */
	
	/* Matrix dimensions */
	CUKR_INT bcsr_nz, bcsr_start_row, bcsr_rows;
	
	/* Row and column vectors */
	CUKR_VECTOR_SP bcsr_ptr, bcsr_idx;
	
	/* Value vector */
	CUKR_VECTOR_SP bcsr_val; 
	 
};


		




				/*****************************************/
				//       CONSTRUCTOR / DESTRUCTOR        //
				/*****************************************/         




/**
 * \brief Free memory all memory used
 *        by the BCSR part of the matrix
 * \param mat The matrix 
 */
void CukrBhybMatspDestroy(CUKR_MATRIX_SP mat); 






				/*****************************************/
				//          TRANSFER / CONVERT           // 
				/*****************************************/ 

  
  
  
  
  

/**
 * \brief Changes the location of a Bhyb matrix
 *        by changing the location of the 
 *        data vectors
 * \param mat The matrix
 * \param dest_loc Destination matrix location (CPU,GPU)
 * \param dest_type Destination matrix value type 
 *        (float, double)
 */
void CukrBhybMatmpTransfer(CUKR_MATRIX_SP mat, int dest_loc, int dest_type);

// ------------------------------------------------------------------- //

/**
 * \brief Changes the storage format of 
 *        the matrix to a diferent variation
 *        or to a completely different format
 * \param mat Pointer to the matrix 
 * \param dest_format Destination format
 * \param dest_variation Destination variation 
 */
void CukrBhybMatspConvert(CUKR_MATRIX_SP mat, int dest_format, int dest_variation);

                




				/*****************************************/
				//               GET/SET                 // 
				/*****************************************/ 






/**
 * \brief Extracts the diagonal from a matrix
 * \param mat The matrix 
 * \param vec The vector where to store the diagonal
 * \param diag The diagonal to read (zero for the
 *        main diagonal)
 * \param invert Whether to invert it or not 
 */
void CukrBhybMatspGetDiag(CUKR_MATRIX_SP mat, CUKR_VECTOR_SP vec, int diag, int invert);







				/*****************************************/
				//                 I/O                   // 
				/*****************************************/ 






/**
 * \brief Prints a matrix to a file stream
 * \param mat The matrix being printed
 * \param f An open file stream to write to
 * \param indent An indentation string, to be inserted
 *        before every print.
 */
void CukrBhybMatspPrintToFileWithIndent(CUKR_MATRIX_SP mat, FILE *f, char* indent);



#endif /*CUKR_HYB_MATRIX_SP_H_*/
