/**
 * @file mat_vec/sp/csr/csr_matrix_sp_gen.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains generators for creating various
 * types of synthetic matrices in CSR format.
 */

#ifndef CUKR_CSR_MATRIX_SP_GEN_H_
#define CUKR_CSR_MATRIX_SP_GEN_H_

#include "../matrix_sp.h"

// ------------------------------------------------------------------- //

/**
 * \brief Generates a sparse random matrix, filled
 *        with random elements, in CSR format 
 * \param mat Pointer to the matrix 
 * \param n Matrix size
 * \param halfb The half-bandwidth of the matrix
 * \param nz_per_row No. of nonzeros per row
 * \param min,max The lower and upper bound for
 *                matrix values
 * \param name Matrix name
 */
void CukrCsrMatspGenRandom(CUKR_MATRIX_SP* mat, int n, int halfb, int nz_per_row, 
						      CUKR_DOUBLE min, CUKR_DOUBLE max, char* name);

// ------------------------------------------------------------------- //
					 
/**
 * \brief Generates a sparse random matrix, filled
 *        with random elements, in CSR format 
 * \param mat Pointer to the matrix 
 * \param n Order
 * \param name Matrix name
 */
void CukrCsrMatspGenPoisson2D(CUKR_MATRIX_SP* mat, int n, char* name);

/**
 * \brief Generates a sparse random matrix, filled
 *        with random elements, in CSR format 
 * \param mat Pointer to the matrix 
 * \param n Order
 * \param name Matrix name
 */
void CukrCsrMatspGenPoisson2Dsingle(CUKR_MATRIX_SP* mat, int n, char* name);

// ------------------------------------------------------------------- //

/**
 * \brief Generates a diagonally-dominated banded
 *        matrix, filled with ones but the main 
 * 		  diagonal computed as such as to be dominant.
 * 		  The diagonals can be shifted to the left or 
 *        right by specifying a negative or positive shift 
 * \param mat Pointer to the matrix 
 * \param format Matrix format (CSR)
 * \param rows*cols Matrix order
 * \param hb Halfbandwidth 
 * \param shift The shift  
 * \param name Matrix name
 */
void CukrCsrMatspGenBandedWShift(CUKR_MATRIX_SP* mat, int rows, int cols, int hb, int shift, char* name);


#endif /*CUKR_CSR_MATRIX_SP_GEN_H_*/
