/**
 * @file kernel_set_dbl_to_val.h
 * @author Serban Georgescu
 * @since 2009
 * 
 * Defines a set value kernel.
 * 
 * The kernel sets all elements of the
 * vector to the specified value.
 * 
 */
 
 
#include "kernels_config.h"

#ifndef _BX_TX
	#define _BX_TX
	#define tx          threadIdx.x 
	#define bx          blockIdx.x
#endif

__global__ void kernel_set_dbl_to_val(      double *d_x, 
                                      const double  val, 
                                      const int     n)
{	
	int i;
				
	/* Starting point for this block */
	int ctaStart = bx * CUKR_SET_DBL_VAL_THREADS; 
	/* Total no. of threads in the kernel */
	int totalThreads = CUKR_SET_DBL_VAL_CTAS * CUKR_SET_DBL_VAL_THREADS;
	
	/* Read and write the data  */
	for (i = ctaStart + tx; i < n; i += totalThreads){		 	 
		d_x[i] = val;
	}  
	 
}

