/**
 * @file kernel_dbl_to_sng.h
 * @author Serban Georgescu
 * @since 2009
 * 
 * Defines a copy kernel.
 * 
 * The kernel copies a single precision data
 * vector to a double precision data vector.
 * 
 */
 
#include "kernels_config.h"

#ifndef _BX_TX
	#define _BX_TX
	#define tx          threadIdx.x 
	#define bx          blockIdx.x
#endif

extern __shared__ float shared_f[];

__global__ void kernel_sng_to_dbl(const float  *d_x, 
                                        double *d_y, 
                                  const int     n)
{	
	int i;
				
	/* Alloc shared memory */
	float *buffer = shared_f; 
	
	/* Starting point for this block */
	int ctaStart = bx * CUKR_COPY_SNG_DBL_THREADS; 
	/* Total no. of threads in the kernel */
	int totalThreads = CUKR_COPY_SNG_DBL_CTAS * CUKR_COPY_SNG_DBL_THREADS;
	
	/* Read and write the data  */
	for (i = ctaStart + tx; i < n; i += totalThreads)
	{  
		/* Prefetch x and y */
		buffer[tx] = d_x[i];
		
		/* Write */		 	 
		d_y[i] = buffer[tx];
	}  
	 
}

