/**
 * @file mat_vec/sp/cuda/vector_kernels.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Acts as a bridge between the C++ CUDA
 * implementation and the rest of the C 
 * files. All the CUDA copy kernels are
 * included here.
 * 
 */

#ifndef CUKR_VECTOR_KERNELS_H_
#define CUKR_VECTOR_KERNELS_H_

/**
 * \brief Dbl->Qdbl kernel on the GPU
 * \param n Vector size
 * \param d_x Device pointer to the source
 *            vector
 * \param d_yh, d_yt Device pointers to the
 *            dest vectors (head and tail)
 */
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_copy_dbl_to_qdbl(int n, double *d_x, float *d_yh, float *d_yt);

// ------------------------------------------------------------------- //

/**
 * \brief Dbl->Sng kernel on the GPU
 * \param n Vector size
 * \param d_x Device pointer to the source
 *            vector
 * \param d_y Device pointer to the dest.
 *            vector
 */
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_copy_dbl_to_sng(int n, double *d_x, float *d_y);

// ------------------------------------------------------------------- //

/**
 * \brief QDbl->Dbl kernel on the GPU
 * \param n Vector size
 * \param d_xh, d_xt Device pointers to the source
 *            vectors (head, tail)
 * \param d_y Device pointer to the dest.
 *            vector
 */
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_copy_qdbl_to_dbl(int n, float *d_xh, float *d_xt, double *d_y);

// ------------------------------------------------------------------- //

/**
 * \brief Sng->Dbl kernel on the GPU
 * \param n Vector size
 * \param d_x Device pointer to the source
 *            vector
 * \param d_y Device pointer to the dest.
 *            vector
 */
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_copy_sng_to_dbl(int n, float *d_x, double *d_y);

// ------------------------------------------------------------------- //

/**
 * \brief Dbl.Vec<-dbl kernel on the GPU
 * \param n Vector size
 * \param d_x Device pointer to the vector 
 * \param val Value to set the vector to
 */
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_set_dbl_to_val(int n, double *d_x, double val);

// ------------------------------------------------------------------- //

/**
 * \brief Sng.Vec <-val kernel on the GPU
 * \param n Vector size
 * \param d_x Device pointer to the vector 
 * \param val Value to set the vector to
 */
#ifdef __cplusplus
extern "C"
#endif 
void cuda_kernel_set_sng_to_val(int n, float *d_x, float val);

#endif /* CUKR_VECTOR_KERNELS_H_ */
