/**
 * @file mat_vec/sp/matrix_sp_gen.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Holds a list of matrix generators.
 * The generators here are indepenent of
 * the matrix format, so the format-specific
 * implementation of the generator will be
 * found elsewere (most probably in the
 * subfolder assigned to the specific format)
 * 
 * The generator will create the matrix,
 * so the matrix should not be created beforehand
 * 
 * Current generators are:
 * 	  - random 
 * 	  - poisson 2D 
 * 
 */

#ifndef CUKR_MATRIX_SP_GEN_H_
#define CUKR_MATRIX_SP_GEN_H_

#include "matrix_sp.h"


/**
 * \brief Generates a diagonally-dominated sparse
 *        random matrix. The matrix is made from
 *        <band_group_size> groups of bands, separated
 *        by <space_group_size> empty bands. There are
 *        a total of group_count groups.  
 * \param mat Pointer to the matrix 
 * \param format Matrix format (CSR)
 * \param n Matrix size
 * \param group_count Number of band groups (symmetric)
 * \param band_group_size How many bands in a group
 * \param space_group_size How many spaces are used
 *        as separators.
 * \param name Matrix name
 */
void CukrMatspGenBanded(CUKR_MATRIX_SP* mat, int format, int n, int group_count, 
                        int band_group_size, int space_group_size, char* name);

#endif /* CUKR_MATRIX_SP_GEN_H_ */
