/**
 * @file monitors/monitors.c
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the monitoring routines. 
 * 
 * Monitoring routines are called inside
 * solvers in order to extract and show
 * various info, like the current value
 * of the residual and so on
 */

#include <math.h>
#include "monitors.h"


/*
 * \brief Displays the current value of
 *        the relative residual 
 * \param solver The solver being monitored
 * \param inner_outer Where is the monitor
 *        called from, the inner solver or
 *        the outer correction iterations
 */
void CukrMonitorRRes(CUKR_SOLVER solver, int inner_outer)
{	
	/* If in inner */
	if (inner_outer == CUKR_INNER)
		printf(" rres = %11.6g ", sqrt(solver->rnorm2) / sqrt(solver->rnorm02));
	/* If in outer */
	else 
		printf(" rres = %11.6g ", sqrt(solver->rnorm2_out) / sqrt(solver->rnorm02_out));
}

/*
 * \brief Displays the current value of
 *        the absolute residual 
 * \param solver The solver being monitored
 * \param inner_outer Where is the monitor
 *        called from, the inner solver or
 *        the outer correction iterations
 */
void CukrMonitorAbsRes(CUKR_SOLVER solver, int inner_outer)
{
	
	/* If in inner */
	if (inner_outer == CUKR_INNER)
		printf(" absres = %11.6g ", sqrt(solver->rnorm2));
	/* if in outer */
	else
		printf(" absres = %11.6g ", sqrt(solver->rnorm2_out));
}
