/**
 * @file monitors/monitors.h
 * @author Serban Georgescu
 * @since 2008
 *
 * Contains the monitoring routines. 
 * 
 * Monitoring routines are called inside
 * solvers in order to extract and show
 * various info, like the current value
 * of the residual and so on
 */

#ifndef CUKR_MONITORS_H_
#define CUKR_MONITORS_H_

#include "../solvers/solver.h"

/*
 * \brief Displays the current value of
 *        the relative residual 
 * \param solver The solver being monitored
 * \param inner_outer Where is the monitor
 *        called from, the inner solver or
 *        the outer correction iterations
 */
void CukrMonitorRRes(CUKR_SOLVER solver, int inner_outer);

/*
 * \brief Displays the current value of
 *        the absolute residual 
 * \param solver The solver being monitored
 * \param inner_outer Where is the monitor
 *        called from, the inner solver or
 *        the outer correction iterations
 */
void CukrMonitorAbsRes(CUKR_SOLVER solver, int inner_outer);

#endif /*CUKR_MONITORS_H_*/
