/**
 * @file monitors/reports/runtime_report.c
 * @author Serban Georgescu
 * @since 2008 
 * 
 * Conains routine for reporting the 
 * perfomance and behaviour of the runtime
 * system
 * 
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "../../util/log.h"
#include "../../db/db.h"
#include "../../blas/blas.h"
#include "../../blasx/blasx.h"
#include "reports.h"

/** Get access to database */
extern CukrDB cukr_db;


// ------------------------------------------------------------------- //

/**
 * \brief Generate runtime report
 * \param filename The name of the file
 *        in which to write the report
 */
void CukrReportRuntime(char* filename)
{
	FILE *fout;
	int i;
	
	/* 
	 * Open the report file 
	 */
	 
	/* If the filename is not null, try to open the file */ 
	if (filename != NULL)
	{	
		if (!cukr_db.mpi_rank)
			if ( (fout = fopen(filename, "w")) == NULL) {
				CUKR_FATALX("Could not open file for reporting: %s\n", filename);
			}
	}
	/* If null, output to stdout */	
	else
	  fout = stdout;
	  
	/*
	 * Write about the runtime behavior
	 */
	
	/* Output only at rank 0 */
	if (!cukr_db.mpi_rank) {
		
		/* Write no. of threads used */
		int pe = CukrDBGetI(CUKR_DB_PE);
		fprintf(fout, "THREADS:%d\n", pe); 
		  
		/* Write the no. of calls to 
		 * CukrThreadsRun */
		 		 				 
		fprintf(fout, "THREADS_RUN_CALLS:%d\n", cukr_db.threadsRunCalls); 
				
		/* Write memory used by all threads */
		fprintf(fout, "HOST_MEMORY:%ld\n", cukr_db.allocated_cpu);
		fprintf(fout, "GPU_MEMORY:%ld\n", cukr_db.allocated_gpu);
		
		/* Close file */
		fclose(fout);		
	}
}
