/**
 * @file solvers/cg_solver.h
 * @author Serban Georgescu
 * @since 2008
 * 
 * Defines the actual implementation
 * of the CG (Conjugate Gradients) 
 * solver. 
 * 
 * The solver is written using the
 * abstract BLAS interface, therefore
 * it runs both on CPU, GPU or using
 * both in combination.
 */

#ifndef CUKR_CG_SOLVER_H_
#define CUKR_CG_SOLVER_H_

#include "../solver.h"

/**
 * Define Jacobi preconditioner data
 * holder format */
typedef struct _CG_SOLVER_DATA {
	/** Hold the temp vectors needed
	 * by the CG solver */
	CUKR_VECTOR r, d, q, s;
} *CG_SOLVER_DATA;

/**
 * \brief Bind the CG solver to the
 *        solver context
 * \param solver The solver context
 */
void solver_create_cg(CUKR_SOLVER solver);
 

#endif /*CUKR_CG_SOLVER_H_*/
