 /**
 * @file log.h
 * @author Serban Georgescu
 * @since 2007
 *
 * This is a wrapper for the logging
 * mechanism provided by the log4.c libray.
 * 
 * We simply wrap the logging routines in
 * macros out of lazyness.
 * 	
 */

#ifndef LOG_H_
#define LOG_H_
	
#include <stdio.h>

/**
 * If the logger name has not been defined,
 * use the root
 */
#ifndef LOGGER_NAME
#define LOGGER_NAME "root"
#endif

/**
 * Wrapper for the Log statament, using the
 * defined LOGGERNAME as the category.
 */
#ifdef CUKR_LOGGING 
	#define LOG(level, msg) {printf("[%s] %s", LOGGER_NAME, msg); printf("\n");} 
#else
	#define LOG(level, msg) 
#endif	

/**
 * Wrapper for the Log statament, printf style
 */
#ifdef CUKR_LOGGING  
	#define LOGX(level, msg, ...)  {char __msg__buffer[255]; sprintf(__msg__buffer, msg, __VA_ARGS__); printf("[%s] %s\n", LOGGER_NAME, __msg__buffer);}
#else
	#define LOGX(level, msg, ...) 
#endif
	
/*
 * Printf then exit
 */
#define CUKR_FATAL(msg) {Log(LOGGER_NAME, FATAL, msg, __FILE__, __LINE__);}

/*
 * Printf then exit, extended version
 */
#define CUKR_FATALX(msg, ...) {char __msg__buffer[255]; sprintf(__msg__buffer, msg, __VA_ARGS__); Log(LOGGER_NAME, FATAL, __msg__buffer, __FILE__, __LINE__);}

/**
 * Redefine the logging priorities, out of 
 * lazyness 
 */
#define FATAL  0
#define ALERT  1
#define CRIT   2
#define ERROR  3
#define WARN   4
#define NOTICE 5
#define INFO   6
#define DEBUG  7
#define TRACE  8
#define NOTSET 9
		

/**
 *  \brief Logging a message to the specified
 *         category.
 * 
 *  \param loggerName the logger name
 *  \param level logging priority
 *  \param msg the actual message
 */
#ifdef __cplusplus
extern "C"
#endif
void
Log(char *loggerName,
     int   level,
     char *msg,
     char *file,
     int line);
 
#endif // LOG_H_
